/* Copyright (c) 2003 ActiveState Corp.
   See the file LICENSE.txt for licensing information. */

/* Dialog to display when license is invalid
 * Basically this works just like the customButtons dialog but has to
 * special layout.
 *
 * Usage:
 *  All dialog interaction is done via an object passed in and out as the first
 *  window argument: window.arguments[0]. All these arguments are optional.
 *      .message        the message to show.
 *      .buttons        list of strings naming buttons to use
 *      .infos          list of strings identifying what information blocks
 *                      to show; valid values are "beta", "purchase",
 *                      "winnt", "aspn"
 *      .response       the default response, must be one of the strings in
 *                      "buttons"
 *      .title          the dialog title
 *  On return window.arguments[0] has:
 *      .response       the name of button used
 */

var log = getLoggingMgr().getLogger("dialogs.invalidLicense");
//log.setLevel(LOG_DEBUG);

var gAcceptButton = null;
var gExtra1Button = null;
var gExtra2Button = null;
var gCancelButton = null;


//---- interface routines for XUL

function OnLoad()
{
    try {
        var i;
        var dialog = document.getElementById("dialog-invalidlicense")
        gAcceptButton = dialog.getButton("accept");
        gExtra1Button = dialog.getButton("extra1");
        gExtra2Button = dialog.getButton("extra2");
        gCancelButton = dialog.getButton("cancel");
        gCancelButton.setAttribute("label", "Close");
        gCancelButton.setAttribute("accesskey", "C");
        var buttonWidgets = [gAcceptButton, gExtra1Button, gExtra2Button];

        // .message
        var messageWidget = document.getElementById("message");
        var message = window.arguments[0].message;
        if (typeof message != "undefined" && message != null) {
            var textNode = document.createTextNode(message);
            messageWidget.appendChild(textNode);
        } else {
            messageWidget.setAttribute("collapsed", "true");
        }

        // .buttons
        var buttons = window.arguments[0].buttons;
        if (typeof buttons == "undefined" || buttons == null) {
            var msg = "Internal Error: illegal 'buttons' value for "
                      +"Custom Buttons dialog: '"+buttons+"'.";
            log.error(msg);
            alert(msg);
            window.close();
        } else if (buttons.length == 0 || buttons.length > 4) {
            var msg = "Internal Error: illegal number of buttons for "
                      +"Custom Buttons dialog: '"+buttons.length+"'.";
            log.error(msg);
            alert(msg);
            window.close();
        } else if (buttons.length == 4 && buttons[3] != "Close") {
            var msg = "Internal Error: there are 4 buttons to display but "
                      +"the last one is not Close.";
            log.error(msg);
            alert(msg);
            window.close();
        } else if (buttons[buttons.length-1] == "Close") {
            for (i = 0; i < buttons.length-1; ++i) {
                buttonWidgets[i].setAttribute("label", buttons[i]);
            }
            for (i = buttons.length-1; i < buttonWidgets.length; ++i) {
                buttonWidgets[i].setAttribute("collapsed", true);
            }
        } else {
            for (i = 0; i < buttons.length; ++i) {
                buttonWidgets[i].setAttribute("label", buttons[i]);
            }
            for (i = buttons.length; i < buttonWidgets.length; ++i) {
                buttonWidgets[i].setAttribute("collapsed", true);
            }
            gCancelButton.setAttribute("collapsed", true);
        }

        // .infos
        var widget;
        var infos = window.arguments[0].infos;
        if (typeof infos == "undefined" || infos == null) infos = [];
        for (i = 0; i < infos.length; ++i) {
            widget = document.getElementById(infos[i]+"-info");
            if (widget && widget.hasAttribute("collapsed")) {
                widget.removeAttribute("collapsed");
            }
        }

        // .response
        var response = window.arguments[0].response;
        if (typeof response == "undefined" || response == null) {
            // gAcceptButton (the first one) is already the default
        } else {
            gAcceptButton.removeAttribute("default");
            for (i = 0; i < buttonWidgets.length; ++i) {
                if (buttonWidgets[i].getAttribute("label") == response) {
                    buttonWidgets[i].setAttribute("default", true);
                    buttonWidgets[i].focus();
                }
            }
            // Note: we are not raising an error or warning if the default response
            // does not match one of the given button names. No big deal.
        }

        // .title
        if (typeof window.arguments[0].title != "undefined" &&
            window.arguments[0].title != null) {
            window.title = window.arguments[0].title;
        } else {
            window.title = "Invalid Komodo License";
        }

        window.sizeToContent();
        // Cannot use dialog.moveToAlertPosition() because that requires
        // a loaded opener window, which this dialog typically does not have.
        dialog.centerWindowOnScreen();
    } catch(ex) {
        log.error(ex, "Error loading invalid license dialog.");
    }
}


function Accept()
{
    try {
        window.arguments[0].response = gAcceptButton.getAttribute("label");
    } catch(ex) {
        log.error(ex);
    }
    return true;
}

function Extra1()
{
    try {
        window.arguments[0].response = gExtra1Button.getAttribute("label");
        // This is one of the "extra" <dialog/> buttons. Only the "accept" and
        // "cancel" button actions will automatically close the window, so we have
        // to do it manually here.
        window.close();
    } catch(ex) {
        log.error(ex);
    }
    return true;
}

function Extra2()
{
    try {
        window.arguments[0].response = gExtra2Button.getAttribute("label");
        // This is one of the "extra" <dialog/> buttons. Only the "accept" and
        // "cancel" button actions will automatically close the window, so we have
        // to do it manually here.
        window.close();
    } catch(ex) {
        log.error(ex);
    }
    return true;
}

function Cancel()
{
    try {
        window.arguments[0].response = gCancelButton.getAttribute("label");
    } catch(ex) {
        log.error(ex);
    }
    return true;
}


