var isearchlog = getLoggingMgr().getLogger('isearch');

var gIsearchController;

function isearch_onload()
{
    gIsearchController = new isearch_isearchController()
    window.controllers.appendController(gIsearchController);
}

function isearch_isearchController(viewManager) {
    this._incrementalSearchPattern = '';
    this._lastIncrementalSearchText = '';
    this.findSvc = Components.classes["@activestate.com/koFindService;1"].
                getService(Components.interfaces.koIFindService);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
isearch_isearchController.prototype = new Controller();
isearch_isearchController.prototype.constructor = isearch_isearchController;

isearch_isearchController.prototype.destructor = function() {
}

isearch_isearchController.prototype.is_cmd_startIncrementalSearch_enabled = function() {
    return gViewMgr.currentView != null;
}

isearch_isearchController.prototype.do_cmd_startIncrementalSearch= function() {
    this._startIncrementalSearch(false);
}

isearch_isearchController.prototype.is_cmd_startIncrementalSearchBackwards_enabled = function() {
    return gViewMgr.currentView != null;
}

isearch_isearchController.prototype.do_cmd_startIncrementalSearchBackwards= function() {
    this._startIncrementalSearch(true);
}

function key_event_handler_for_isearch(e) {
    gIsearchController.keyPressForSearch(e); // doesn't take into account focus shifts in other scintillas
}

function mouse_event_handler_for_isearch(e) {
    // If called from <scintilla>, 'this' will be the XBL widget.  Weird JS.
    if (this.key_handler) {
        this.key_handler = null;
    }
    if (this.mouse_handler) {
        this.mouse_handler = null;
    }
    this.removeEventListener('blur', mouse_event_handler_for_isearch, false);

    gIsearchController._stopIncrementalSearch("Clicked away", false);
}

isearch_isearchController.prototype._startIncrementalSearch = function(backwards) {
    var scintilla = gViewMgr.currentView.scintilla; // XXX fixme?
    if (!scintilla) {
        isearchlog.error("Couldn't start incremental search with no focused scintilla");
        return;
    }
    var scimoz = scintilla.scimoz;
    StatusBar_AddMessage("Incremental Search:", "isearch", 0, false, true)
    scintilla.key_handler = key_event_handler_for_isearch;
    scintilla.mouse_handler = mouse_event_handler_for_isearch;
    scintilla.addEventListener('blur', mouse_event_handler_for_isearch, false);
    this._incrementalSearchStartPos = Math.min(scimoz.currentPos, scimoz.anchor);
    var pattern = scimoz.selText;
    this._incrementalSearchContext = Components.classes["@activestate.com/koFindContext;1"]
        .createInstance(Components.interfaces.koIFindContext);

    // Save original find settings
    this._origFindOptions = new Object();
    this._origFindOptions.searchBackward = this.findSvc.options.searchBackward;
    this._origFindOptions.matchWord= this.findSvc.options.matchWord;
    this._origFindOptions.patternType = this.findSvc.options.patternType;
    this._origFindOptions.caseSensitivity = this.findSvc.options.caseSensitivity;

    this.findSvc.options.searchBackward = backwards;
    this.findSvc.options.matchWord = false;
    this.findSvc.options.patternType = Number(gPrefs.getStringPref('isearchType'));
    this.findSvc.options.caseSensitivity = Number(gPrefs.getStringPref('isearchCaseSensitivity'));
    this._incrementalSearchContext.type = this.findSvc.options.FCT_CURRENT_DOC;
    this._incrementalSearchPattern = pattern;
    if (pattern) {
        // we don't want to find the current selection, dummy.
        scimoz.currentPos = this._incrementalSearchStartPos + pattern.length;
        this._lastIncrementalSearchText = pattern;
        StatusBar_AddMessage("Incremental Search: " + pattern, "isearch", 0, false, true)
        Find_FindNext(window, this._incrementalSearchContext,
                      pattern, null, true,
                      true);  // useMRU: add this pattern to the find MRU
    }
}

isearch_isearchController.prototype._stopIncrementalSearch = function(why, highlight) {
    StatusBar_AddMessage("Incremental Search Stopped: " + why, "isearch", 3000, highlight, true)
    if (this._origFindOptions) {
        // Restore original find settings
        this.findSvc.options.searchBackward = this._origFindOptions.searchBackward;
        this.findSvc.options.matchWord = this._origFindOptions.matchWord;
        this.findSvc.options.patternType = this._origFindOptions.patternType;
        this.findSvc.options.caseSensitivity = this._origFindOptions.caseSensitivity;
        this._origFindOptions = null;
    }
    MRU_add("find-patternMru", this._incrementalSearchPattern, true);
    this._incrementalSearchPattern = '';
}

isearch_isearchController.prototype.is_cmd_rawKey_enabled = function() {
    return gViewMgr.currentView != null;
}

isearch_isearchController.prototype.do_cmd_rawKey= function() {
    var scintilla = gViewMgr.currentView.scintilla;
    scintilla.key_handler = this.rawHandler;
    scintilla.addEventListener('blur', gCancelRawHandler, false);
    scintilla.scimoz.focus = true;
    StatusBar_AddMessage("Enter Control Character:", "raw_input", 0, true, true)
}

function gCancelRawHandler(event) {
    if (this.key_handler) {
        this.key_handler = null;
    }
    StatusBar_AddMessage(null, "raw_input", 0, false, true)
}

isearch_isearchController.prototype.rawHandler= function(event) {
    try {
        if (event.type != 'keypress') return;
        var scintilla = gViewMgr.currentView.scintilla;
        scintilla.key_handler = null;
        var scimoz = scintilla.scimoz;
        event.cancelBubble = true;
        event.preventDefault();
        // XXX handle meta key here?
        if (event.ctrlKey) {
            // Need to convert from charCode to ASCII value
            scimoz.replaceSel(String.fromCharCode(event.charCode-96));
        } else {
            if (event.charCode != 0) {  // not sure why space doesn't work.
                scimoz.replaceSel(String.fromCharCode(event.charCode));
            } else {
                switch (event.keyCode) {
                    case event.DOM_VK_ESCAPE:
                    case event.DOM_VK_ENTER:
                    case event.DOM_VK_RETURN:
                    case event.DOM_VK_TAB:
                    case event.DOM_VK_BACK_SPACE:
                        scimoz.replaceSel(String.fromCharCode(event.keyCode));
                        break;
                    default:
                        // do nothing
                }
            }
        }
        StatusBar_AddMessage(null, "raw_input", 0, false, true)
    } catch (e) {
        log.error(e);
    }
}

isearch_isearchController.prototype.do_cmd_repeatNextCommandBy= function() {
    try {
        var scintilla = gViewMgr.currentView.scintilla;
        scintilla.key_handler = this.multiHandler;
        scintilla.addEventListener('blur', gCancelMultiHandler, false);
        scintilla.scimoz.focus = true;
        gIsearchController.inRepeatCounterAccumulation = true;
        gIsearchController.repeatCounter = 0;
        StatusBar_AddMessage("Number of Repeats: |", "multi_input", 0, true, true)
    } catch (e) {
        log.exception(e);
    }
}

function gCancelMultiHandler(event) {
    gIsearchController.inRepeatCounterAccumulation = false;
    gViewMgr.currentView.scintilla.key_handler = null;
    StatusBar_AddMessage(null, "multi_input", 0, false, true)
}

isearch_isearchController.prototype.multiHandler= function(event) {
    try {
        if (event.type != 'keypress') return;
        if (event.charCode >= 48 && event.charCode < 57) {
            gIsearchController.repeatCounter = gIsearchController.repeatCounter * 10 + (event.charCode - 48);
            StatusBar_AddMessage("Number of Repeats: " + String(gIsearchController.repeatCounter) + '|',
                                 "multi_input", 0, false, true)
            return;
        }
        var key = gKeybindingMgr.event2keylabel(event);
        // If the key corresponds to the cmd_cancel command, cancel.
        if (gKeybindingMgr.command2key['cmd_cancel'] == key) {
            event.cancelBubble = true;
            gCancelMultiHandler();
            return;
        }
        if (event.charCode && !event.ctrlKey && !event.altKey && !event.metaKey) {
            gCancelMultiHandler();
            // it's just a simple keystroke, do that.
            key = String.fromCharCode(event.charCode);
            var txt = '';
            for (var i = 0; i < gIsearchController.repeatCounter; i++) {
                txt += key;
            }
            var scintilla = gViewMgr.currentView.scintilla;
            scintilla.scimoz.replaceSel(txt);
        }
    } catch (e) {
        log.error(e);
    }
}

isearch_isearchController.prototype.keyPressForSearch = function(event) {
    try {
        if (event.type != 'keypress') return;
        var v = gViewMgr.currentView;
        var scintilla = v.scintilla;
        var scimoz = v.scintilla.scimoz;
        var pos = scimoz.selectionStart < scimoz.selectionEnd ? scimoz.selectionStart : scimoz.selectionEnd;
        var eventkey = gKeybindingMgr.event2keylabel(event);
        var triggerkeys = gKeybindingMgr.command2keysequences('cmd_startIncrementalSearch');
        var backwardstriggerkeys = gKeybindingMgr.command2keysequences('cmd_startIncrementalSearchBackwards');

        // If the key sequence associated with incremental search is more than
        // one key long (i.e. the user has customized from the default <Ctrl+I>
        // to, say, <Ctrl+K, I>):
        // - if the last key is a Ctrl or Alt key, use it
        // - otherwise, disable the 'multiple hits' feature
        var triggerkey;
        if (triggerkeys) {
            triggerkey = triggerkeys[triggerkeys.length-1];
        }
        var backwardstriggerkey = '';
        if (backwardstriggerkeys) {
            backwardstriggerkey = backwardstriggerkeys[backwardstriggerkeys.length-1];
        }
        if ((triggerkeys.length == 1 
             || triggerkey.indexOf('Ctrl') != -1
             || triggerkey.indexOf('Meta') != -1
             || triggerkey.indexOf('Alt') != -1)
            && (eventkey == triggerkey || eventkey == backwardstriggerkey))
        {
            // we're hitting the same key as was used to start the search,
            // so we just want to redo the search
            event.cancelBubble = true;
            event.preventDefault();
            if (this._lastIncrementalSearchText == '')
                return;
            if (this._incrementalSearchPattern == "") {
                // Second <Ctrl+I> after a non-search action
                this._incrementalSearchPattern = this._lastIncrementalSearchText;
                gFindSession.Reset();
            }
            if (eventkey == backwardstriggerkey) {
                this.findSvc.options.searchBackward = true;
            } else {
                this.findSvc.options.searchBackward = false;
            }
            var findres = Find_FindNext(window, this._incrementalSearchContext,
                                        this._incrementalSearchPattern,
                                        null, true,
                                        true); // add pattern to find MRU
            if (findres == false) {
                StatusBar_AddMessage("Incremental Search: No more occurences of "
                    + this._incrementalSearchPattern +' found',
                    "isearch", 0, false, true)
            } else {
                this._incrementalSearchStartPos = scimoz.currentPos;
                StatusBar_AddMessage("Incremental Search: "
                    + this._incrementalSearchPattern,
                    "isearch", 0, false, true)
            }
            return;
        }

        var isShiftLeftKey = event.shiftKey && event.keyCode == event.DOM_VK_LEFT;
        var isShiftRightKey = event.shiftKey && event.keyCode == event.DOM_VK_RIGHT;
        var isBS = event.keyCode == event.DOM_VK_BACK_SPACE;
        var isRegularKey = event.keyCode == 0;
        var isSlash = event.keyCode == event.DOM_VK_SLASH;
        if (!event.ctrlKey
            && !event.altKey && !event.metaKey 
            && (isRegularKey || isShiftRightKey || isShiftLeftKey || isBS || isSlash))
        {
            var pattern = this._incrementalSearchPattern;
            event.cancelBubble = true;
            event.preventDefault();
            if (isShiftRightKey) {
                var nextChar = scimoz.getWCharAt(scimoz.currentPos);
                pattern = pattern + nextChar;
                event.stopPropagation();  // We don't want normal shift-arrow behavior to occur
            } else if (isShiftLeftKey || isBS) {
                if (pattern == '') return; // nothing left to backtrack
                pattern = pattern.substr(0, pattern.length-1);
                event.stopPropagation();  // We don't want normal shift-arrow behavior to occur
            } else if (isSlash) {
                // do nothing;  The "real" slash charCode-bearing event will come later.
                event.stopPropagation();
                return;
            } else {
                pattern = pattern + String.fromCharCode(event.charCode);
            }
            this._lastIncrementalSearchText = pattern;
            this._incrementalSearchPattern = pattern;
            var oldStart = scimoz.selectionStart;
            var oldEnd = scimoz.selectionEnd;
            scimoz.gotoPos(this._incrementalSearchStartPos-1);
            if (this._incrementalSearchPattern == '') {
                StatusBar_AddMessage("Interactive Search: |", "isearch",
                                     0, false, true);
                return;
            }
            gMacroMgr.undo();
            findres = Find_FindNext(
                window, this._incrementalSearchContext,
                this._incrementalSearchPattern, null,
                true,
                // Do NOT add this pattern to find MRU
                //  http://bugs.activestate.com/Komodo/show_bug.cgi?id=27350
                // that will be done on stopping of interactive search.
                false);
            if (! findres) {
                StatusBar_AddMessage("Interactive Search: No occurences of "
                    + this._incrementalSearchPattern + " found.",
                    "isearch", 3000, true, true);
                scimoz.setSel(oldStart, oldEnd);
            } else {
                StatusBar_AddMessage("Interactive Search: "
                    + this._incrementalSearchPattern + '|',
                    "isearch", 0, false, true)
            }
            return;
        }
        else {
            scintilla.key_handler = null;
            scintilla.mouse_handler = null;
            scintilla.removeEventListener('blur',
                mouse_event_handler_for_isearch, false);
            // Anything, not just escape, cancels incremental search
            this._stopIncrementalSearch("Search canceled.");
            var key = gKeybindingMgr.event2keylabel(event);
            if (gKeybindingMgr.command2key['cmd_cancel'] == key) {
                event.cancelBubble = true;
            }
            return;
        }
    } catch (e) {
        isearchlog.error(e);
    }
}

