/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
var gPeMacro = new peMacro();
registeredExtensions[registeredExtensions.length] = gPeMacro;

function peMacro() {
    this.name = 'peMacro';
    this._hookedMacros = [];
    this.observer = new peMacroEventObserver();
    var obsSvc = Components.classes["@mozilla.org/observer-service;1"].
                       getService(Components.interfaces.nsIObserverService);
    obsSvc.addObserver(this.observer, 'macro-load', false);
    obsSvc.addObserver(this.observer, 'macro-unload', false);
    obsSvc.addObserver(this.observer, 'javascript_macro',false);
    obsSvc.addObserver(this.observer, 'part-invoke',false);
    obsSvc.addObserver(this.observer, 'command-docommand',false);
    this.log = getLoggingMgr().getLogger('peMacro');
    //this.log.setLevel(LOG_DEBUG);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peMacro.prototype.constructor = peMacro;

peMacro.prototype.init = function() {
}

peMacro.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_executeMacro', this);
}

peMacro.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart_macro,
                                     'ondblclick', this);
    extensionManager.addEventHandler(Components.interfaces.koIPart_macro,
                                     'onUpdatePart', this);
}

peMacro.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_macro,
                                    'Execute Macro',
                                    'cmd_executeMacro',
                                    null,
                                    null,
                                    true);
}

peMacro.prototype.ondblclick = function(item,event) {
    if (item.type != 'macro') return;
    macro_executeMacro(item.part)
}

peMacro.prototype.supportsCommand = function(command, item) {
    switch (command) {
    case 'cmd_executeMacro':
        if (gFocusedProjectView == null) return false;
        var items = gFocusedProjectView.getSelectedItems();
        if (items.length == 1 && items[0].part
            && items[0].part.type == 'macro') {
            return true;
        } else {
            return false;
        }
    default:
        break;
    }
    return false;
}

peMacro.prototype.isCommandEnabled = peMacro.prototype.supportsCommand;

function macro_ranking(mac1, mac2) {
    if (! mac1.hasAttribute('rank')) return 1;
    if (! mac2.hasAttribute('rank')) return -1;
    return mac1.getLongAttribute('rank') < mac2.getLongAttribute('rank');
}

peMacro.prototype.callHookedMacros = function(trigger) {
    if (! gPrefs.getBooleanPref("triggering_macros_enabled")) {
        return false;
    }
    this._hookedMacros.sort(macro_ranking);
    var macro;
    for (var i = 0; i < this._hookedMacros.length; i++) {
        macro = this._hookedMacros[i];
        if (macro.hasAttribute('trigger_enabled') &&
            macro.getBooleanAttribute('trigger_enabled') &&
            macro.getStringAttribute('trigger') == trigger) {
            if (macro_executeMacro(macro)) {
                return true;
            }
        }
    }
    return false;
}

peMacro.prototype.addMacro = function(macropart) {
    // A macro has been added -- we should find out if it
    // has any 'hooks'
    for (var i = 0; i < this._hookedMacros.length; i++) {
        if (this._hookedMacros[i] == macropart) {
            this.log.error("Couldn't add macro " + macropart.id + " to list of hooked macros: it was already in the list.");
            return;
        }
    }
    this._hookedMacros.push(macropart);
}

peMacro.prototype.removeMacro = function(macropart) {
    // A macro has been removed -- we should remove it from
    // the list of hooks
    try {
        for (var i = 0; i < this._hookedMacros.length; i++) {
            if (this._hookedMacros[i].id == macropart.id) {
                this._hookedMacros.splice(i, 1); // remove it
                return;
            }
        }
    } catch (e) {
        this.log.exception(e);
    }
    this.log.error("Couldn't remove macro from list of hooked macros.");
}

peMacro.prototype.hookOnStartup = function() {
    return this.callHookedMacros('trigger_startup');
}

peMacro.prototype.hookPostFileOpen = function() {
    return this.callHookedMacros('trigger_postopen');
}

peMacro.prototype.hookPreFileSave = function() {
    return this.callHookedMacros('trigger_presave');
}

peMacro.prototype.hookPostFileSave = function() {
    return this.callHookedMacros('trigger_postsave');
}

peMacro.prototype.hookPreFileClose = function() {
    return this.callHookedMacros('trigger_preclose');
}

peMacro.prototype.hookPostFileClose = function() {
    return this.callHookedMacros('trigger_postclose');
}

peMacro.prototype.hookOnQuit = function() {
    return this.callHookedMacros('trigger_quit');
}

function peMacroEventObserver() {};
peMacroEventObserver.prototype.observe = function(part, topic, code)
{
    try {
        switch (topic) {
            case 'macro-load':
                gPeMacro.addMacro(part);
                break;
            case 'macro-unload':
                gPeMacro.removeMacro(part);
                break;
            case 'javascript_macro':
                // dump("part = " + part + "\n");
                macro_evalAsJavascript(code);
                break;
            case 'part-invoke':
                //dump("got part-invoke: " + part + '\n' + dummy + '\n');
                part_invokePart(part);
                break;
            case 'command-docommand':
                //dump("got command-docommand: " + code + '\n');
                command_doCommand(code);
                break;
        };
    } catch (e) {
        log.exception(e);
    }
}

function peMacro_addMacro(item)
{
    var part = Components.classes['@activestate.com/koPart?type=macro;1']
               .createInstance(Components.interfaces.koIPart_macro);
    part.setStringAttribute('name', 'New Macro');
    var obj = new Object();
    obj.item = new partWrapper(part, 0);
    obj.parent = item;
    obj.task = 'new';
    openWindowUniqueInstance(
        "chrome://jaguar/content/project/macroProperties.xul",
        "komodo_macroProperties",
        "chrome,centerscreen,dialog=no,close=yes,dependent=yes,resizable=yes", obj);
}

function macro_executeMacro(part)
{
    gPeMacro.log.info("macro_executeMacro part.id:"+part.id);
    try {
        macro_recordPartInvocation(part);
        language = part.getStringAttribute('language').toLowerCase();
        var async = part.getBooleanAttribute('async');
        if (language == 'javascript' && async) {
            window.setTimeout('macro_executeMacroById("' + part.id +
                              '", ' + async + ')', 10);
        } else {
            return _executeMacro(part, async)
        }
    } catch (e) {
        gPeMacro.log.exception(e);
    }
    return false;
}


function macro_executeMacroById(id, asynchronous) {
    try {
        var macroPart = gPartSvc.getPartById(id);
        retval = _executeMacro(macroPart, asynchronous);
        return retval;
    } catch (e) {
        gPeMacro.log.exception(e);
    }
    return false;
}

function _executeMacro(part, asynchronous) {
    // Returns true if there was a problem running the macro
    // The synchronous flag is not used for JavaScript, since the timeout
    // has already occurred by the time we get called here.  JS execution
    // isn't in another thread, just on a timeout.
    try {
        gMacroMgr.suspendRecording();
        var language = part.getStringAttribute('language').toLowerCase();
        var retval = false;
        var exception = null;
        var view = gViewMgr.currentView;
        var editor = null;
        if (view && view.getAttribute('type') == 'editor' && view.scintilla && view.scintilla.scimoz) {
            editor = view.scintilla.scimoz;
        }
        var code = stringutils_unescapeWhitespace(part.value, '\n');
        switch (language) {
            case 'javascript':
                try {
                    retval = macro_evalAsJavascript(code, part);
                } catch (e) {
                    exception = String(e);
                }
                break;
            case 'python':
                try {
                    view = gViewMgr.currentView;
                    var doc = null;
                    if (view && view.document) {
                        doc = view.document;
                    }
                    editor = null;
                    if (view && view.getAttribute('type') == 'editor' && view.scimoz) {
                        editor = view.scimoz;
                    }
                    retval = part.evalAsPython(document, window, editor,
                                               doc, view,
                                               code, asynchronous);
                } catch (e) {
                    log.exception(e);
                    var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                                       getService(Components.interfaces.koILastErrorService);
                    exception = lastErrorSvc.getLastErrorMessage();
                }
                break;
            default:
                retval = "Macros written in '"+language+"' aren't supported."
        }
        if (exception) {
            dialog_alert("There was a problem executing the macro: " +
                         part.getStringAttribute('name'), exception);
            return true;
        } else {
            return retval;
        }
    } catch (e) {
        gPeMacro.log.exception(e);
    }
    gMacroMgr.resumeRecording();
    return false;
}

function assertMacroVersion(version) {
    // Here's where we could do something like the following -- which isn't actually
    // needed since the macros from 2.3 appear to work in 2.5 with the three
    // aliases defined in macro_evalAsJavascript
    //if (version != "2") {
    //    throw "Macros from previous versions of Komodo are not supported in this version of Komodo.";
    //}
}

function macro_findPart(type, name, where) {
    var _partSvc = Components.classes["@activestate.com/koPartService;1"]
            .getService(Components.interfaces.koIPartService);
    return _partSvc.findPartForRunningMacro(type, name, where);
}


function macro_evalAsJavascript(code, part /* = null */) {
    try {
        if (typeof(part) == 'undefined') {
            part = null;
        }
        var view = null;
        if (gViewMgr.currentView) {
            view = gViewMgr.currentView
        }

        var komodo = new _KomodoJSMacroAPI(part, view);
        eval("function _code() { " + code + " }; ");
        retcode = eval("_code();");
        komodo.destructor();
        delete komodo;

        return retcode;
    } catch (e) {
        log.exception(e);
    }
    return false;
}

peMacro.prototype.doCommand = function(command) {
    var cmd = null;
    var item = null;
    switch (command) {
    case 'cmd_executeMacro':
        item = gFocusedProjectView.getSelectedItem();
        macro_executeMacro(item.part);
        break;
    default:
        break;
    }
}

function macro_editProperties(item)
{
    var obj = new Object();
    obj.item = item;
    obj.task = 'edit';
    openWindowMultipleInstance(
        "chrome://jaguar/content/project/macroProperties.xul",
        "komodo_macroproperties",
        "chrome,centerscreen,close=yes,dialog=no,dependent=yes,resizable=yes", obj);
}


peMacro.prototype.onUpdatePart = function(item,event)
{
    if (item.type == 'macro') {
        item.renameable = true;
        item.iconstyle['macro'] = 1;
    }
}

function macro_recordPartInvocation(part) {
    try {
        // We'll do the macro work ourselves using the name of the part
        // instead of the encoded contents of the run command -- the
        // resulting macro has a _reference_ to the run command, which
        // seems more representative of what the user expects (and means
        // that the run command can be developed independently of the
        // macro.
        var win = windowManager_getMainWindow();
        if (typeof(win.gMacroMgr) != 'undefined' &&
            win.gMacroMgr &&
            win.gMacroMgr.mode == 'recording') {
            var name = part.getStringAttribute('name');
            var type = part.type;
            var runtxt = "_part = komodo.findPart('"+type+"', '" + name + "'" + ", '*')\n";
            runtxt += "if (!_part) {alert(\"Couldn't find a " + type + " called '" + name + "' when executing macro.\"); return\n}\n";
            runtxt += "_part.invoke();\n";
            win.gMacroMgr.appendCode(runtxt);
        }
    } catch (e) {
        log.exception(e);
    }
}


//---- Komodo JS Macro Object

function _KomodoJSMacroAPI(macro, view)
{
    log.debug("_KomodoJSMacroAPI()");
    try {
        this.doc = null;
        this.document = null;
        if (view && view.document) {
            this.doc = view.document;
            this.document = this.doc;
        }
        this.editor = null;
        if (view && view.scimoz) {
            this.editor = view.scimoz;
        }
        this.view = view;
        this.macro = macro;

        this.window = window;
        this.domdocument = document;
        this.components = Components;
        var _partSvc = Components.classes["@activestate.com/koPartService;1"]
                .getService(Components.interfaces.koIPartService);
        _partSvc.runningMacro = macro;

    } catch(ex) {
        log.exception(ex);
    }
}
_KomodoJSMacroAPI.prototype.constructor = _KomodoJSMacroAPI;

_KomodoJSMacroAPI.prototype.destructor = function() {
    delete this.editor;
    delete this.view;
    delete this.window;
    delete this.document;
    delete this.domdocument;
}

_KomodoJSMacroAPI.prototype.assertMacroVersion = assertMacroVersion;
_KomodoJSMacroAPI.prototype.doCommand = command_doCommand;
_KomodoJSMacroAPI.prototype.findPart = macro_findPart;

_KomodoJSMacroAPI.prototype.getWordUnderCursor = function()
{
    return Interpolate_getWordUnderCursor(this.editor);
}

_KomodoJSMacroAPI.prototype.interpolate = function(s, bracketed /*=false*/)
{
    if (typeof(bracketed) == "undefined") bracketed = false;
    log.debug("_KomodoJSMacroAPI.interpolate('"+s+"', bracketed="+bracketed+")");
    var istring = null;

    //XXX We are assuming that Interpolate_interpolate() gathers
    //    viewData for the same view for this same view for which this
    //    macro is being run. Currently in Komodo this is the current
    //    view for both so we are fine.
    var bracketedStrings, strings;
    if (bracketed) {
        strings = [];
        bracketedStrings = [s];
    } else {
        strings = [s];
        bracketedStrings = [];
    }
    var queryTitle = null;
    if (this.macro) {
        queryTitle = "Macro '"+this.macro.getStringAttribute("name")+"' Query";
    }
    try {
        var mainWindow = windowManager_getMainWindow();
        var istrings = Interpolate_interpolate(mainWindow, strings,
                                               bracketedStrings,
                                               queryTitle);
        istring             = istrings[0];
        //istringForDisplay = istrings[1];
    } catch (ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"]
                           .getService(Components.interfaces.koILastErrorService);
        var errno = lastErrorSvc.getLastErrorCode();
        if (errno == Components.results.NS_ERROR_ABORT) {
            // interpolation was cancelled.
        } else if (errno == Components.results.NS_ERROR_INVALID_ARG) {
            var errmsg = lastErrorSvc.getLastErrorMessage();
            throw("could not interpolate string: " + errmsg);
        } else {
            throw(ex);
        }
    }
    return istring;
}


