/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peMenu();
var prefSvc = Components.classes['@activestate.com/koPrefService;1'].
                getService(Components.interfaces.koIPrefService);

function peMenu() {
    try {
        this.name = 'peMenu';
        this.log = getLoggingMgr().getLogger('peMenu');
        this.observer = new peMenuEventObserver();
        var obsSvc = Components.classes["@mozilla.org/observer-service;1"].
                           getService(Components.interfaces.nsIObserverService);
        obsSvc.addObserver(this.observer, 'menu_create', false);
        obsSvc.addObserver(this.observer, 'menu_changed', false);
        obsSvc.addObserver(this.observer, 'menu_remove', false);
        obsSvc.addObserver(this.observer, 'toolbar_create', false);
        obsSvc.addObserver(this.observer, 'toolbar_remove', false);
        obsSvc.addObserver(this.observer, 'toolbar_changed', false);
        obsSvc.addObserver(this.observer, 'part_changed', false);
    } catch (e) {
        this.log.exception(e);
    }
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peMenu.prototype.constructor = peMenu;

peMenu.prototype.init = function() {
}

peMenu.prototype.registerCommands = function() {
}

peMenu.prototype.registerEventHandlers = function() {
    try {
        extensionManager.addEventHandler(Components.interfaces.koIPart_menu,
                                         'onUpdatePart',this);
        extensionManager.addEventHandler(Components.interfaces.koIPart_toolbar,
                                         'onUpdatePart',this);
        extensionManager.registerCommand('cmd_viewCustomToolbar', this);
    } catch (e) {
        this.log.exception(e);
    }
}

peMenu.prototype.registerMenus = function() {
    var menupopup = document.getElementById('folder_context');
    extensionManager.addMenuItem(Components.interfaces.koIPart_menu, menupopup);
}

peMenu.prototype.ondblclick = function(part,event) {
}

peMenu.prototype.supportsCommand = function(command, part) {
    if (command == 'cmd_viewCustomToolbar') return true;
    return false;
}

peMenu.prototype.isCommandEnabled = peMenu.prototype.supportsCommand;

peMenu.prototype.doCommand = function(command) {
    if (command == 'cmd_viewCustomToolbar') {
        // which toolbar is being changed?
        //uilayout_toggleToolbarVisibility('standardToolbar');
    }
}


peMenu.prototype.onUpdatePart = function(item,event)
{
    try {
        if (item.type == 'menu') {
            item.iconopenstyle['menu'] = 1
            item.iconstyle['menu'] = 1
        } else {
            item.iconopenstyle['toolbar'] = 1
            item.iconstyle['toolbar'] = 1
        }
        item.iconopenstyle['projectFolderOpen'] = 0;
        item.iconstyle['projectFolderClosed'] = 0;
    } catch (e) {
        this.log.exception(e);
    }
}

function __getPartName(part) {
    var name, url;
    if (part.hasAttribute('name')) {
        name = part.getStringAttribute('name');
    } else if (part.hasAttribute('url')) {
        url = part.getStringAttribute('url');
        name = uriparse_baseName(url);
    }
    return name;
}

function peMenuEventObserver() {};
peMenuEventObserver.prototype.observe = function(part, topic, dummy)
{
    try {
        var menu;
        switch (topic) {
            case 'toolbar_remove':
                peMenu_removeToolbarForPart(part);
                break;
            case 'toolbar_changed':
                peMenu_removeToolbarForPart(part);
                // intentionaly fall through
            case 'toolbar_create':
                peMenu_addToolbarFromPart(part);
                break;
            case 'menu_remove':
                try {
                    // First get rid of the old menu
                    menu = document.getElementById(part.id);
                    if (!menu) {
                        log.warning("Couldn't delete menu with id: " + part.id);
                        break;
                    }
                    menu.parentNode.removeChild(menu);
                } catch (e) {
                    log.exception(e);
                }
                break;
            case 'part_changed':
                try {
                    var xulelt = document.getElementById(part.id);
                    if (xulelt) {
                        xulelt.setAttribute('image', part.iconurl)
                        name = __getPartName(part);
                        xulelt.setAttribute('label', name)
                    }
                } catch (e) {
                    log.exception(e);
                }
                break;
            case 'menu_changed':
                try {
                    // First get rid of the old menu
                    menu = document.getElementById(part.id);
                    if (!menu) {
                        log.error("Couldn't find menu with id: '" + part.id + "'");
                        break;
                    }
                    menu.parentNode.removeChild(menu);
                    peMenu_addMenuFromPart(part);
                } catch (e) {
                    log.exception(e);
                }
                break;
            case 'menu_create':
                peMenu_addMenuFromPart(part);
                break;
        }
    } catch (e) {
        log.exception(e);
    }
};

function peMenu_partAcceptsMenuToolbar(part) {
    // Used to check early whether a part can be added
    while (part && part.type != 'project') {
        if (part.type == 'menu') return false;
        if (part.type == 'toolbar') return false;
        part = part.parent;
    }
    return true;
}

function peMenu_addMenu(item) {
    // Ensure that the item being added to isn't a menu/toolbar or a
    // child of a menu/toolbar
    if (item && ! peMenu_partAcceptsMenuToolbar(item.part)) {
        dialog_alert("Sorry, you can't add menus to menus, toolbars or children "+
                     "of menus or toolbars. If you want to add a sub-menu "+
                     "add a Folder item.");
        return;
    }
    name = dialog_prompt("Enter menu name:");
    if (!name) return;
    var index = gFocusedProjectView.getIndexByPart(item);
    var menu = Components.classes["@activestate.com/koPart?type=menu;1"]
                                    .createInstance(Components.interfaces.koIPart_menu);
    menu.name = name;
    gFocusedProjectView.manager.addItem(menu,index);
}

function peMenu_addMenuFromPart(part) {
    try {
        var base_ordinal = 100;

        var name = __getPartName(part);
        var menubar = document.getElementById('menubar_main')
        var menu = document.createElement('menu');
        menu.setAttribute('label', name);
        var accesskey = part.getStringAttribute('accesskey');
        if (accesskey) {
            menu.setAttribute('accesskey', accesskey);
        }
        menu.setAttribute('id', part.id);
        var menupopup = document.createElement('menupopup');
        menu.appendChild(menupopup);
        menu.ordinal = base_ordinal + part.getLongAttribute('priority');
        menubar.appendChild(menu);

        _peMenu_fillMenupopupFromPart(menupopup, part);
    } catch (e) {
        log.exception(e);
    }
}

function peMenu_removeToolbarForPart(part) {
    try {
        // First get rid of the old menu
        var toolbar = document.getElementById(part.id);
        if (!toolbar) {
            log.error("Couldn't find toolbar with id: '" + part.id + "'");
            return;
        }
        toolbar.parentNode.removeChild(toolbar)

        var menuitem = document.getElementById('context_custom_toolbar_'+part.id)
        menuitem.parentNode.removeChild(menuitem);

        menuitem = document.getElementById('menu_custom_toolbar_'+part.id)
        menuitem.parentNode.removeChild(menuitem);

        var broadcaster = document.getElementById('cmd_custom_toolbar_'+part.id)
        broadcaster.parentNode.removeChild(broadcaster);

        var context = document.getElementById("context-toolbox-menu");
        var menu = document.getElementById("popup_toolbars");
        if (context.lastChild.nodeName == 'menuseparator') {
            context.lastChild.setAttribute('collapsed', 'true');
            menu.lastChild.setAttribute('collapsed', 'true');
        }

    } catch(e) {
        log.exception(e);
    }
}

function peMenu_addToolbarFromPart(part) {
    try {
        // uncollapse the seperator if necessary
        var context = document.getElementById("context-toolbox-menu");
        var menu = document.getElementById("popup_toolbars");
        if (context.lastChild.nodeName == 'menuseparator') {
            context.lastChild.removeAttribute('collapsed');
            menu.lastChild.removeAttribute('collapsed');
        }

        // minimal display ordinal an element will have
        var base_ordinal = 100;
        var cmd_id = 'cmd_custom_toolbar_'+part.id;
        var visible = ! peMenu_isToolbarRememberedAsHidden(part.id);

        var toolbox = document.getElementById('toolbox_main');
        var toolbar = document.createElement('toolbar');
        toolbar.setAttribute('id', part.id);
        toolbar.setAttribute('class', "chromeclass-toolbar");
        // We need the last custom toolbox to have a flex of one
        toolbar.setAttribute('buttonstyle', "pictures");
        toolbar.setAttribute('grippyhidden', "true");
        toolbar.setAttribute('broadcaster', cmd_id);
        if (! visible) {
            toolbar.setAttribute('hidden', 'true');
        }
        var ordinal = base_ordinal + part.getLongAttribute('priority');
        toolbar.ordinal = ordinal;

        toolbox.appendToolbar(toolbar);

        _peMenu_fillToolbarFromPart(toolbar, part);

        // append the toolboar to the toolbar context and view->toolbar menues
        var broadcasterset = document.getElementById("broadcasterset_global");
        var broadcaster = document.createElement("broadcaster");
        //<broadcaster
        //    id="cmd_viewedittoolbar"
        //    autoCheck="false"
        //    checked="true"
        //    persist="checked"
        //    oncommand="uilayout_toggleToolbarVisibility('standardToolbar')"/>
        broadcaster.setAttribute('id', cmd_id);
        broadcaster.setAttribute('autoCheck', 'false');
        broadcaster.setAttribute('persist', 'checked');
        if (visible) {
            broadcaster.setAttribute('checked', 'true');
        } else {
            broadcaster.setAttribute('checked', 'false');
        }
        broadcaster.setAttribute('oncommand', 'uilayout_toggleToolbarVisibility("'+part.id+'"); peMenu_toggleToolbarHiddenStateInPref("' + part.id + '")');
        broadcasterset.appendChild(broadcaster);


        var menuitem = document.createElement('menuitem');
        //<menuitem id="context_viewStandardToolbar"
        //          label="Standard Toolbar"
        //          type="checkbox"
        //          observes="cmd_viewedittoolbar"
        //          key="key_viewToolbarEdit"/>
        menuitem.setAttribute('id', 'context_custom_toolbar_'+part.id);
        menuitem.setAttribute('label', __getPartName(part));
        menuitem.setAttribute('type', 'checkbox');
        if (visible) {
            menuitem.setAttribute('checked', 'true');
        } else {
            menuitem.setAttribute('checked', 'false');
        }
        menuitem.setAttribute('observes', cmd_id);
        //menuitem.setAttribute('oncommand', 'uilayout_toggleToolbarVisibility("'+part.id+'");');
        menuitem.ordinal = ordinal;
        context.appendChild(menuitem);

        //<menuitem label="Standard"
        //          id="menu_viewedittoolbar"
        //          observes="cmd_viewedittoolbar"
        //          persist="checked"
        //          checked="true"
        //          type="checkbox"
        //          />
        menuitem = document.createElement('menuitem');
        menuitem.setAttribute('id', 'menu_custom_toolbar_'+part.id);
        menuitem.setAttribute('label', __getPartName(part));
        menuitem.setAttribute('type', 'checkbox');
        menuitem.setAttribute('persist', 'checkbox');
        if (visible) {
            menuitem.setAttribute('checked', 'true');
        } else {
            menuitem.setAttribute('checked', 'false');
        }
        menuitem.setAttribute('observes', cmd_id);
        //menuitem.setAttribute('oncommand', 'uilayout_toggleToolbarVisibility("'+part.id+'");');
        menuitem.ordinal = ordinal;
        menu.appendChild(menuitem);


    } catch (e) {
        log.exception(e);
    }
}

function peMenu_isToolbarRememberedAsHidden(id) {
        try {
        var prefs = prefSvc.prefs
        if (!prefs.hasStringPref('hidden_toolbars')) {
            return false;
        }
        var hidden_toolbar_ids = prefs.getStringPref('hidden_toolbars')
        var ids = hidden_toolbar_ids.split(',');
        var found = false;
        for (var i = 0; i < ids.length; i++) {
            if (ids[i] == id) {
                return true;
            }
        }
    } catch (e) {
        log.exception(e);
    }
    return false;
}
function peMenu_toggleToolbarHiddenStateInPref(id) {
    try {
        var prefs = prefSvc.prefs
        var hidden_toolbar_ids = '';
        if (prefs.hasStringPref('hidden_toolbars')) {
            hidden_toolbar_ids = prefs.getStringPref('hidden_toolbars')
        }
        var ids = hidden_toolbar_ids.split(',');
        var found = false;
        for (var i = 0; i < ids.length; i++) {
            if (ids[i] == id) {
                // take it out
                ids.splice(i,1);
                found = true;
                break;
            }
        }
        if (!found) { // must add it instead
            ids.push(id);
        }
        hidden_toolbar_ids = ids.join(',');
        prefs.setStringPref('hidden_toolbars', hidden_toolbar_ids)
    } catch (e) {
        log.exception(e);
    }
}

function peMenu_addToolbar(item)
{
    // Ensure that the item being added to isn't a menu/toolbar or a
    // child of a menu/toolbar
    if (item && ! peMenu_partAcceptsMenuToolbar(item.part)) {
        dialog_alert("Sorry, you can't add toolbars to menus, toolbars or children of menus or toolbars.");
        return;
    }
    name = dialog_prompt("Enter toolbar name:");
    if (!name) return;
    var index = gFocusedProjectView.getIndexByPart(item);
    var toolbar = Components.classes["@activestate.com/koPart?type=toolbar;1"]
                                    .createInstance(Components.interfaces.koIPart_toolbar);
    toolbar.name = name;
    gFocusedProjectView.manager.addItem(toolbar,index);
}


function _peMenu_createToolbaritemFromPart(toolbar, part)
{
  try {
    var button, name, id;
    button = document.createElement('toolbarbutton');
    if (! gUILayout_buttonTextShowing) {
        button.setAttribute('buttonstyle', 'pictures');
    }

    name = __getPartName(part);
    switch (part.type) {
        case 'ProjectRef':
            // same as file
        case 'file':
            url = part.getStringAttribute('url');
            button.setAttribute('oncommand', "try { open_openURI('" + url + "') } catch (e) { log.exception(e); };");
            break;
        case 'dialog':
            url = part.getStringAttribute('url');
            name = uriparse_baseName(url);
            button.setAttribute('oncommand', "try { peGUI_editDialog('" + url + "') } catch (e) { log.exception(e); };");
            break;
        case 'folder':
            button.setAttribute('type', 'menu');
            button.setAttribute('class', 'rightarrow-button-a');
            button.setAttribute('orient', 'horizontal');
            menupopup = document.createElement('menupopup');
            button.appendChild(menupopup);
            var i, childitem;
            var children = new Array();
            part.getChildren(children, new Object());
            children = children.value;
            children.sort(_peMenu_sortItems);
            for (i = 0; i < children.length; i++) {
                childitem = _peMenu_createMenuItemFromPart(menupopup,children[i]);
                menupopup.appendChild(childitem);
            }
            break;
        default:
            button.setAttribute('oncommand', "part_invokePartById('" + part.id + "');");
            break;
        }
    button.setAttribute('image', part.iconurl);
    button.setAttribute('tooltiptext', name);
    button.setAttribute('tooltip', 'aTooltip');
    button.setAttribute('label', name);
    button.setAttribute('id', part.id);
    return button;
  } catch (e) {
    log.exception(e);
  }
  return null;
}

function _peMenu_createMenuItemFromPart(menupopup, part)
{
    var menuitem, name;
    switch (part.type) {
        case 'ProjectRef':
            // same as file
        case 'file':
            url = part.getStringAttribute('url');
            name = __getPartName(part);
            menuitem = document.createElement('menuitem');
            menuitem.setAttribute('class', 'menuitem-iconic-wide');
            menuitem.setAttribute('oncommand', "try { open_openURI('" + url + "') } catch (e) { log.exception(e); };");
            break;
        case 'dialog':
            url = part.getStringAttribute('url');
            name = uriparse_baseName(url);
            menuitem = document.createElement('menuitem');
            menuitem.setAttribute('class', 'menuitem-iconic-wide');
            menuitem.setAttribute('oncommand', "try { peGUI_editDialog('" + url + "') } catch (e) { log.exception(e); };");
            break;
        case 'folder':
            name = __getPartName(part);
            menuitem = document.createElement('menu');
            menuitem.setAttribute('class', 'menu-iconic');
            menupopup = document.createElement('menupopup');
            menuitem.appendChild(menupopup);
            var i, childitem;
            var children = new Array();
            part.getChildren(children, new Object());
            children = children.value;
            children.sort(_peMenu_sortItems);
            for (i = 0; i < children.length; i++) {
                childitem = _peMenu_createMenuItemFromPart(menupopup,children[i]);
                menupopup.appendChild(childitem);
            }
            break;
        default:
            name = __getPartName(part);
            menuitem = document.createElement('menuitem');
            menuitem.setAttribute('class', 'menuitem-iconic-wide');
            menuitem.setAttribute('oncommand', "part_invokePartById('" + part.id + "');");
            break;
        }
    menuitem.setAttribute('id', part.id);
    menuitem.setAttribute('image', part.iconurl);
    menuitem.setAttribute('label', name);
    if (part.hasAttribute('keyboard_shortcut')) {
        var shortcut = part.getStringAttribute('keyboard_shortcut');
        if (shortcut) {
            menuitem.setAttribute('acceltext', shortcut);
        }
    }
    return menuitem;
}

function _peMenu_fillMenupopupFromPart(menupopup, part)
{
    var children = new Array();
    part.getChildren(children, new Object());
    children = children.value;
    children.sort(_peMenu_sortItems);
    var name, url, child, menuitem;
    for (i = 0; i < children.length; i++) {
        child = children[i];
        menuitem = _peMenu_createMenuItemFromPart(menupopup, child);
        menupopup.appendChild(menuitem);
    }
}

function _peMenu_fillToolbarFromPart(toolbar, part)
{
    var children = new Array();
    part.getChildren(children, new Object());
    var button;
    children = children.value;
    children.sort(_peMenu_sortItems);
    for (i = 0; i < children.length; i++) {
        part = children[i];
        button = _peMenu_createToolbaritemFromPart(toolbar, part);
        toolbar.appendChild(button);
    }
}

function _peMenu_sortItems(a,b) {
    // sort folders first
    if (a.type == 'folder' && b.type !='folder')
        return -1;
    if (a.type != 'folder' && b.type =='folder')
        return 1;
    // sort other types alphabeticaly
    if (a.type != b.type) {
        if (a.type < b.type)
            return -1;
        else if (a.type > b.type)
            return 1;
    }
    if (a.hasAttribute('name') && b.hasAttribute('name')) {
        var aname = a.getStringAttribute('name').toLowerCase();
        var bname = b.getStringAttribute('name').toLowerCase();
        if (aname < bname )
            return -1;
        else if (aname > bname )
            return 1;
    }
    return 0;
}

function peMenu_editProperties(item, rank) {
    var obj = new Object();
    obj.item = item;
    obj.task = 'edit';
    obj.imgsrc = 'chrome://jaguar/skin/images/open.png';
    if (item.type == 'menu') {
        obj.type = 'menu';
        obj.prettytype = 'Custom Menu';
    } else {
        obj.type = 'toolbar';
        obj.prettytype = 'Custom Toolbar';
    }
    openWindowMultipleInstance(
        "chrome://jaguar/content/project/customMenuProperties.xul",
        "komodo_customMenuProperties",
        "chrome,close=yes,dependent=yes,resizable=yes", obj);
}

