/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: JavaScript; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * projectmanager.js
 * Handle Komodo Project Files -- main part
 *
 * To be loaded in the same .xul file as projectviewer.js
 */

// Globals
var gProjectManager = null;

//----- The projectManager class manages the set of open projects and
// which project is 'current'.

function projectManager() {
    gTimeline.enter('projectManager()');
    this.name = 'projectManager';
    baseManager.apply(this, ["projectsChanged"]);
    this.log = getLoggingMgr().getLogger('projectManager');
    this._projects = [];
    this._currentProject = -1;
    this.viewMgr = document.getElementById('projectview');
    if (!this.viewMgr) {
        log.error("couldnt' find id 'projectview'");
        return;
    }
    this.viewMgr.onLoad(this);
    gPartManagers.push(this);
    // Make sure that there is always a gFocusedProjectView
    gFocusedProjectView = this.viewMgr.view
    // register our command handlers
    this.registerCommands();
    // add our default datapoint
    this.viewMgr.addColumns(extensionManager.datapoints);
    extensionManager.datapoints['Name']='Name';



    this.lastsorttype = null;
    this.sorttype = 'Name';
    this.sortdir = 1;
    this.firstsort = true;
    gTimeline.leave('projectManager()');
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
projectManager.prototype = new baseManager();
projectManager.prototype.constructor = projectManager;

projectManager.prototype.getProjectsMenu = function(popup)
{
    // used by top level project->set active menu
    var curProject = this.getCurrentProject();
    var children = popup.childNodes;
    var i,mi;
    for (i = 0; i < children.length; i++) {
        popup.removeChild(children[i]);
    }
    if (this._projects.length > 0) {
        for (var project in this._projects) {
            mi = document.createElementNS(XUL_NS, 'menuitem');
            mi.setAttribute('label',this._projects[project].name);
            mi.setAttribute('oncommand','gProjectManager.setCurrentProject('+project+');');
            mi.setAttribute('type','checkbox');
            if (curProject == this._projects[project]) {
                mi.setAttribute('checked','true');
            }
            popup.appendChild(mi);
        }
    } else {
        // add an 'empty' menu item
        mi = document.createElementNS(XUL_NS, 'menuitem');
        mi.setAttribute('label','No Open Projects');
        mi.setAttribute('disabled','true');
        popup.appendChild(mi);
    }
    return true;
}

projectManager.prototype.updateMenuForInterface = function(defaultMenuInterface,menuEl) {
    extensionManager.updateMenuForInterface(null, menuEl);
}

projectManager.prototype.closeProjectByURL = function(url) {
    for (var i = 0; i < this._projects.length; i++) {
        if (this._projects[i].part.url == url) {
            return this.closeProject(i);
        }
    }
    return false;
}






projectManager.prototype._getOpenURLsInProject = function(project) {
    // Find out if any child elements are currently open
    var docs = gViewMgr.topView.getDocumentViewList(true);
    var opened;



    opened = [];

    var url;
    for (var i = 0; i < docs.length; i++) {
        if (docs[i].document && docs[i].document.file) {
            url = docs[i].document.file.URI;
            if (project.part.getChildByURL(url)) {
                opened.push(url);
            }
        }
    }
    return opened;
}

projectManager.prototype.forcedCloseURL = function(url) {
    var v = gViewMgr.getViewForFile(url);
    if (v) {
        // we don't want dialogs to popup here!!!
        v.closeUnconditionally();
    }





}


projectManager.prototype._saveProjectViewState = function(project) {
    // This function goes through all of the URLs in a project,
    // finds out if any of them are "open", and saves the
    // list in a viewState Pref for the Project's KPF file.

    var viewStateMRU = gPrefSvc.getPrefs("viewStateMRU");
    var projectViewState;
    var url = project.part.url;
    var opened_files = Components.classes['@activestate.com/koOrderedPreference;1'].createInstance();
    var some_opened_files = false;
    var v;

    if (viewStateMRU.hasPref(url)) {
        projectViewState = viewStateMRU.getPref(url);
    } else {
        projectViewState = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
        projectViewState.id = url
        viewStateMRU.setPref(projectViewState);
    }
    var urls = this._getOpenURLsInProject(project);
    for (var i=0; i < urls.length; i++) {
        opened_files.appendStringPref(urls[i]);
    }
    projectViewState.setPref("opened_files", opened_files);
}

projectManager.prototype.closeProjectEvenIfDirty = function(id) {
    var project = this._projects[id];

    if (gCodeIntelActive)
        gCodeIntelSvc.ideEvent("closing_project", null, project.part);

    // Determine the row of the top-level project node for the project
    // being closed.
    var treeView = this.viewMgr.view;
    var parentIndex;
    var closedProjectRow = treeView.mSelection.currentIndex;
    var sentinel = 5;
    while (closedProjectRow != -1 && sentinel) {
        parentIndex = treeView.getParentIndex(closedProjectRow);
        if (parentIndex == -1) break;
        closedProjectRow = parentIndex;
        sentinel--;
    }

    // Remove the project node/part from the Projects tree.
    this.viewMgr.view.closeToplevelPart(id);

    // Reset the active project and tree selection.
    var newActiveProjectIndex = null;  // null means "don't change it"
    var newTreeRow = null;
    if (treeView.entriesList.length == 0) {
        newActiveProjectIndex = -1;
        newTreeRow = -1;
    } else {
        var nearByProjectIndex;
        if (id+1 < this._projects.length) {
            nearByProjectIndex = id+1
            newTreeRow = closedProjectRow; // should be row of next project
        } else {
            nearByProjectIndex = id-1;
            newTreeRow = closedProjectRow-1; // start at child of preceding project
            sentinel = 5;
            while (newTreeRow != -1 && sentinel) {
                parentIndex = treeView.getParentIndex(newTreeRow);
                if (parentIndex == -1) break;
                newTreeRow = parentIndex;
                sentinel--;
            }
        }
        if (id == this._currentProject) { // active project is being closed
            newActiveProjectIndex = nearByProjectIndex;
        }
    }
    if (newActiveProjectIndex != null)
        this.setCurrentProject(newActiveProjectIndex);
    if (newTreeRow != null)
        treeView.mSelection.select(newTreeRow);

    // remove the project from our list
    this._projects.splice(id, 1);
    project.part.close();
    if (this._currentProject >= id) { // adjust active project index
        this._currentProject--;
    }

    MRU_addURL("mruProjectList", project.part.url);
    project.destroy();
    if (this._projects.length == 0) {
        window.updateCommands('some_projects_open');
    }
    return true;
}

projectManager.prototype.closeProject = function(id) {
    var project = this._projects[id];

    if (project.part.isDirty) {
        var question = "Save changes to project '"+project.name+"'?";
        var answer = dialog_yesNoCancel(question);
        if (answer == "Cancel") {
            return false;
        } else if (answer == "Yes") {
            try {
                if (! id in this._projects) {
                    this.log.error("can't save unknown project with id " + id);
                    return false;
                }
                project = this._projects[id];
                this.saveProject(project);
            } catch(ex) {
                var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                    getService(Components.interfaces.koILastErrorService);
                dialog_alert('There was an error saving project, "' +
                      project.name + '": ' +
                      lastErrorSvc.getLastErrorMessage());
                return false;
            }
        }
    }
    this._saveProjectViewState(project);
    var urls = this._getOpenURLsInProject(project);
    if (urls.length != 0) {
        var action = dialog_yesNoCancel(
                "Would you like Komodo to close the opened files from "+
                    "this project?",
                "No", null, null, // default response, text, title
                "close_all_files_on_project_close");
        if (action == "Cancel") {
            return false;
        } else if (action == "Yes") {
            // Should find out which ones are dirty and offer to save those --
            // then _closeURL can be brutal with those that the user didn't
            // want to save.
            var modifieds = new Array();
            var i;
            modified = gViewMgr.offerToSave(urls, "Save Modified Files", "Save selected files before closing them?");
            if (modified == false) return false;

            for (i=0; i < urls.length; i++) {
                this.forcedCloseURL(urls[i]);
            }
        }
    }
    return this.closeProjectEvenIfDirty(id);
}

projectManager.prototype.getDirtyProjectURLs= function() {
    var dirtyURLs = new Array();
    for (var i = 0; i < this._projects.length; i++) {
        if (this._projects[i].part.isDirty) {
            dirtyURLs.push(this._projects[i].part.url);
        }
    }
    return dirtyURLs;
}

projectManager.prototype.getDirtyProjectIDs= function() {
    var dirtyIDs = new Array();
    for (var i = 0; i < this._projects.length; i++) {
        if (this._projects[i].part.isDirty) {
            dirtyIDs.push(i);
        }
    }
    return dirtyIDs;
}

projectManager.prototype.closeAllProjects = function() {
    for (var i = 0; i < this._projects.length; i++) {
        if (!this.closeProject(i)) return false;
    }
    return true;
}

projectManager.prototype.closeAllProjectsEvenIfDirty = function() {
    for (var i = 0; i < this._projects.length; i++) {
        if (!this.closeProjectEvenIfDirty(i)) return false;
    }
    return true;
}

projectManager.prototype.saveProject = function(project) {
    // Returns true on success, false on failure.
    var lastErrorSvc;

































    if (project.properties['isReadOnly']) {
        alert("The project '" + project.name +
              "' is a read only file and cannot be saved.");
        return false;
    } else {
        try {
            project.part.save();
        } catch(ex) {
            lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                getService(Components.interfaces.koILastErrorService);
            dialog_alert('There was an error saving project "' +
                  project.name + '": ' +
                  lastErrorSvc.getLastErrorMessage());
            return false;
        }
        // invalidate so the dirty status shows correctly
        this.viewMgr.view.mTree.invalidate();
        try {
            gObserverSvc.notifyObservers(this, 'file_changed', project.url);
        } catch(e) { /* exception if no listeners */ }
        window.updateCommands('project_dirty');
    }
    return true;
}

projectManager.prototype.newProject = function(url) {
    var project = Components.classes["@activestate.com/koProject;1"]
                                        .createInstance(Components.interfaces.koIProject);
    project.create();
    project.url = url;
    try {
        project.save();
    } catch(ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        dialog_alert('There was an error saving project "' +
              project.name + '": ' +
              lastErrorSvc.getLastErrorMessage());
        return false;
    }
    this.loadProject(url);
    try {
        gObserverSvc.notifyObservers(self, 'file_project', url);
    } catch(e) { /* exception if no listeners */ }
    if (gCodeIntelActive) {
        // We delay launching the code browser for a little while to provide a
        // smoother project-opening experience. It's not a big deal that the
        // code browser gets populated a tad late.
        window.setTimeout(
            function(codeIntelSvc, msg, url, project) {
                codeIntelSvc.ideEvent(msg, url, project);
            },
            1000, gCodeIntelSvc, "opened_project", url, project
        );
    }
    return true;
}

projectManager.prototype.revertProjectByURL = function(url) {
    this._revertProject(this.getProjectIdByURL(url),url);
}

projectManager.prototype.revertProjectById = function(id) {
    this._revertProject(id,this.getProjectById(id).url);
}

projectManager.prototype._revertProject = function(id,url) {
    this.closeProjectEvenIfDirty(id);
    this.loadProject(url);
}

projectManager.prototype.loadProject = function(url) {
    var project = Components.classes["@activestate.com/koProject;1"]
                        .createInstance(Components.interfaces.koIProject);

    // is the project already loaded?
    for (var i = 0; i < this._projects.length; i++) {
        if (this._projects[i].url == url) {
            return null;
        }
    }
    window.setCursor("wait");
    try {
        this.log.info("loading url: " + url);
        project.load(url);
    } catch(e) {
        window.setCursor("auto");
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        var projectname;
        try {
            projectname = uriparse_URIToLocalPath(url);
        } catch (ex) {
            projectname = url;
        }
        if (!projectname) {  // XXX Is this case cruft? I think so. --TM
            projectname = url;
        }
        dialog_alert("Unable to load project \"" + projectname + "\": " +
              lastErrorSvc.getLastErrorMessage());
        return null;
    }

    var wrapped = new partWrapper(project, 0);
    wrapped.url = url;
    this._projects[this._projects.length] = wrapped;
    this.setCurrentProject(this._projects.length-1);

    // add project to project tree
    this.firstsort = true;
    this.sort();
    // Open the project by default -- this feels like it belongs elsewhere... XXX
    this.viewMgr.view.toggleOpenState(this.viewMgr.view.getIndexByPart(wrapped),true);

    // let the status service know it has work to do
    try {
        gObserverSvc.notifyObservers(self, 'file_update_now', null);
    } catch(e) { /* exception if no listeners */ }

    MRU_addURL("mruProjectList", url);
    window.setCursor("auto");
    this.viewMgr.focus(); // always set focus here when loading projects
    window.updateCommands('some_projects_open');
    return project;
}

projectManager.prototype.sort = function(sorttype) {
    if (this._projects.length < 1) return;
    var currentProjectName = null;
    if (this._projects.length > 1) {
        // ALWAYS sort the projects by name
        currentProjectName = this._projects[this._currentProject].name;
        var olddir = partwrapper_sortDirection;
        partwrapper_sortDirection = 1;
        var oldtype = partwrapper_sortField;
        partwrapper_sortField = 'Name';
        this._projects.sort(compareWrappers);
        partwrapper_sortField = oldtype;
        partwrapper_sortDirection = olddir;
    }
    if (typeof(sorttype) != 'undefined') {
        this.sorttype = sorttype;
    }
    if (!this.firstsort) {
        if (this.sorttype == this.lastsorttype) {
            this.sortdir = this.sortdir * -1;
        } else {
            // reset the direction since we're sorting
            // on a different column
            this.sortdir = 1;
        }
        this.viewMgr.view.updateSortIndicators(this.sorttype);
    }
    this.firstsort = false;
    partwrapper_sortDirection = this.sortdir;
    partwrapper_sortField = this.lastsorttype = this.sorttype;

    // clear the list and reinsert the projects
    this.viewMgr.view.clearList();
    for (var i in this._projects) {
        this._projects[i].sortChildren();
        if (currentProjectName &&
            this._projects[i].name == currentProjectName)
            this._currentProject = i;
    }
    this.viewMgr.view.addToplevelChildren(this._projects);
}

projectManager.prototype.getProjectByItem = function(item) {
    for (var i in this._projects) {
        var project = this.isChildOf(this._projects[i], item);
        if (project != null) return project;
    }
    return null;
}

projectManager.prototype.getProjectByURL = function(url) {
    for (project in this._projects) {
        if (project.url == url) {
            return project;
        }
    }
    return null;
}

projectManager.prototype.getProjectIdByURL = function(url) {
    var id, project;
    for (id in this._projects) {
        project = this._projects[id];
        if (project.url == url) {
            return id;
        }
    }
    return -1;
}
projectManager.prototype.getProjectById = function(projectId) {
    return this._projects[projectId];
}

projectManager.prototype.getCurrentProjectId = function() {
    return this._currentProject;
}

projectManager.prototype.getCurrentProject = function() {
    if (this._currentProject >=0 ) return this._projects[this._currentProject];
    return null;
}

projectManager.prototype.getSelectedProjectId = function() {
    var projectId = this.viewMgr.view.getSelectedToplevelPartId();
    if (projectId < 0) projectId = this.getCurrentProjectId();
    return projectId;
}

projectManager.prototype.getPartById = function (id) {
    // return the part with specified id if exists in our projects.
    var item;
    for (var i in this._projects) {
        item = this._projects[i].getPartById(id);
        if (item != null) return item;
    }
    return null;
}

projectManager.prototype.setCurrentProject = function(projectIndex) {
    if (this._currentProject >= 0) {
        this._projects[this._currentProject].iconstyle['projectActive'] = 0;
        this._projects[this._currentProject].iconopenstyle['projectActive'] = 0;
    }
    this._currentProject = projectIndex;
    if (this._currentProject >= 0) {
        this._projects[this._currentProject].iconstyle['projectActive'] = 1;
        this._projects[this._currentProject].iconopenstyle['projectActive'] = 1;
        if (gCodeIntelActive) {
            gCodeIntelSvc.ideEvent("current_project_changed", null,
                               this._projects[this._currentProject].part);
        }
    }
    this.refreshView();
    window.updateCommands('current_project_changed');
}

projectManager.prototype.registerCommands = function() {
    extensionManager.registerCommand("cmd_newProject",this);
    extensionManager.registerCommand("cmd_openProject",this);
    extensionManager.registerCommand("cmd_openProjectFromURL",this);
    extensionManager.registerCommand("cmd_setActiveProject",this);
    extensionManager.registerCommand("cmd_closeProject",this);
    extensionManager.registerCommand("cmd_saveProject",this);
    extensionManager.registerCommand("cmd_saveProjectAs",this);
    extensionManager.registerCommand("cmd_revertProject",this);
    extensionManager.registerCommand("cmd_importPackageToToolbox",this);

    extensionManager.createMenuItem(Components.interfaces.koIProject,
                                    'Make Active Project','cmd_setActiveProject');
    extensionManager.createMenuItem(Components.interfaces.koIProject,
                                    'Close Project','cmd_closeProject');
    extensionManager.createMenuItem(Components.interfaces.koIProject,
                                    'Save Project','cmd_saveProject');
    extensionManager.createMenuItem(Components.interfaces.koIProject,
                                    'Save Project As...','cmd_saveProjectAs');
    extensionManager.createMenuItem(Components.interfaces.koIProject,
                                    'Revert Project','cmd_revertProject');
    extensionManager.createMenuItem(Components.interfaces.koIToolbox,
                                    'Import Package into Toolbox...','cmd_importPackageToToolbox');
}

projectManager.prototype.supportsCommand = function(command, item) {
    // avoid startup confusion
    if (!gFocusedProjectView) return false;

    switch(command) {
    case "cmd_setActiveProject":
    case "cmd_newProject":
    case "cmd_openProject":
    case "cmd_openProjectFromURL":
    case "cmd_closeProject":
    case "cmd_saveProjectAs":
    case "cmd_importPackageToToolbox":
        return true;
    case "cmd_revertProject":
    case "cmd_saveProject":
        var nitem = gFocusedProjectView.getSelectedItem();
        if (!nitem || nitem.type != 'project') {
            nitem = this.getCurrentProject();
        }
        return (nitem && nitem.part && nitem.part.isDirty);
    default:
        return false;
    }
}

projectManager.prototype.isCommandEnabled = function(command) {
    switch(command) {
    case "cmd_setActiveProject":
        return this.getCurrentProjectId() != this.viewMgr.view.getSelectedToplevelPartId();
        break;
    case "cmd_newProject":
    case "cmd_openProject":
    case "cmd_importPackageToToolbox":
        return true;
    case "cmd_closeProject":
        var broadcaster = document.getElementById('broadcaster_projectCurrent');
        if (this._projects.length == 0) {
            broadcaster.setAttribute('disabled', 'true');
        } else if (broadcaster.hasAttribute('disabled')) {
            broadcaster.removeAttribute('disabled');
        }
        return this.getCurrentProjectId() >= 0;
    case "cmd_saveProject":
    case "cmd_revertProject":
        var item = gFocusedProjectView.getSelectedItem();
        if (!item || item.type != 'project') {
            item = this.getCurrentProject();
        }
        return (item && item.part && item.part.isDirty);
    case "cmd_saveProjectAs":
        var projectId = this.viewMgr.view.getSelectedToplevelPartId();
        if (projectId < 0) projectId = this._currentProject;
        return projectId >= 0;
    }
    return false; // shutup strict js
}

projectManager.prototype.doCommand = function(command) {
    var filename, uri;
    var project;
    var projectId = this.getSelectedProjectId();
    switch(command) {
    case "cmd_setActiveProject":
        this.setCurrentProject(projectId);
        break;
    case "cmd_newProject":
        filename = filepicker_saveFile(
            null, // defaultDir
            "MyProject.kpf", // defaultFilename
            "New Project", // title
            "Komodo Project", // defaultFilterName
            ["Komodo Project", "All"]); // filterNames
        if (filename == null) return;
        uri = uriparse_localPathToURI(filename);
        this.newProject(uri);
        break;
    case "cmd_openProject":
        var defaultDirectory = null;
        var defaultFilename = null;
        var title = "Open Project";
        var defaultFilterName = 'Komodo Project';
        var filterNames = ['Komodo Project', 'All'];
        filename = filepicker_openFile(defaultDirectory /* =null */,
                             defaultFilename /* =null */,
                             title /* ="Open File" */,
                             defaultFilterName /* ="All" */,
                             filterNames /* =null */)
        if (filename == null) return;
        uri = uriparse_localPathToURI(filename);
        project_openProjectFromURL(uri);
        break;
    case "cmd_closeProject":
        this.closeProject(projectId);
        break;
    case "cmd_saveProject":
        if (! projectId in this._projects) {
            this.log.error("can't save unknown project with id " + projectId);
            return;
        }
        project = this._projects[projectId];
        this.saveProject(project);
        break;
    case "cmd_revertProject":
        this.revertProjectById(projectId);
        break;
    case "cmd_saveProjectAs":
        if (projectId < 0) return;
        project = gProjectManager.getProjectById(projectId);
        ProjectSaveAs(project);
        break;
    case "cmd_importPackageToToolbox":
        Toolbox_ImportPackage();
        break;
    }
    return;
}

projectManager.prototype.removeItem = function(item, skipdialog) {
    if (baseManager.prototype.removeItem.apply(this, [item, skipdialog])) {
        window.updateCommands('project_dirty');
        return true;
    }
    return false;
}

projectManager.prototype.addItem = function(/* koIPart */ part, index) {
    if (typeof(index)=='undefined') index = -1;
    var isprojectref = false;
    if (index < 0) {
        try {
            /* if the target is not a project, then open it */
            isprojectref = part.QueryInterface(Components.interfaces.koIPart_ProjectRef) != null;
        } catch(e) {}
        if (isprojectref) {
            open_openURI(part.getStringAttribute('url'));
            return false; // return false so drag-move doesn't remove the original part
        }
    }
    var isproject = false;
    try {
        /* we don't allow projects to be added to
           projects */
        isproject = part.QueryInterface(Components.interfaces.koIProject) != null;
        if (isproject) return false;
    } catch(e) {}

    try {
        var parent = null;
        if (index >= 0 && (this.viewMgr.view.entriesList[index].level > 0 || this.viewMgr.view.entriesList[index].isContainer)) {
            if (!this.viewMgr.view.entriesList[index].isContainer) {
                index = this.viewMgr.view.getParentIndex(index);
            }
            parent = this.viewMgr.view.entriesList[index];
        } else {
            parent = this.getCurrentProject();
        }
        // if the url is in the project already, quit
        if (part.hasAttribute('url')) {
            var url = part.getStringAttribute('url');
            if (parent.part.getChildWithTypeAndStringAttribute(part.type,'url',url,false))
                return false;
        }
        if (part.hasAttribute('name')) {
            if (parent.part.getChildWithTypeAndStringAttribute(part.type,'name',part.getStringAttribute('name'),false))
                return false;
        }
        parent.part.addChild(part);
        var firstchild = parent.updateWrappedPart();
        this.viewMgr.view.refresh(parent);
        this.viewMgr.view.selectPart(firstchild);
        window.updateCommands('project_dirty');
        return true;
    } catch(e) { }
    return false;
}


projectManager.prototype.getItemsByURL = function(url, type) {
    var items = [];
    var item;
    for (var i in this._projects) {
        item = this.findItemByURLInProject(this._projects[i], type, url);
        if (item != null) items.push(item);
    }
    return items;
}

projectManager.prototype.getPartsByURL = function(url) {
    var part;
    var parts = [];
    for (var i in this._projects) {
        part = this._projects[i].part.getChildByAttributeValue('url', url, true);
        if (part != null) parts.push(part);
    }
    return parts;
}

/* We may need to optimize this if these functions end up being called a lot
  Currently they're only called when the GUI builder creates new files */

projectManager.prototype.findItemByURL = function(url) {
    for (var i in this._projects) {
        var item = this.findItemByURLInProject(this._projects[i], null, url);
        if (item != null) return item;
    }
    return null;
}

projectManager.prototype.findItemByURLInProject = function (project, type, url) {
    var item;
    var child;
    var i;
    var j;
    if (project.hasWrappedChildren()) {
        for (i in project.children) {
            item = project.children[i];
            if (!type || item.type == type) {
                if (item.url == url || (item.part.hasAttribute('url') && (item.part.getStringAttribute('url') == url))) {
                    return item;
                }
            }
            if (item.hasWrappedChildren()) {
                for (j in item.children) {
                    child = this.findItemByURLInProject(item.children[j], type, url);
                    if (child != null) {
                        return child;
                    }
                }
            }
        }
    }
    if (project.url == url || (project.part.hasAttribute('url')
            && project.part.getStringAttribute('url') == url)) {
        return project;
    }
    return null;
}

projectManager.prototype.findItemByAttributeValue = function(attribute, value) {
    var item;
    for (var i in this._projects) {
        item = this.findChildByAttributeValue(this._projects[i], attribute, value);
        if (item != null) return item;
    }
    return null;
}

projectManager.prototype.findPartByTypeAttributeValue= function (type, attribute, value) {
    var part;
    var cp = this.getCurrentProjectId();
    if (cp >= 0) {
        part = this._projects[cp].part.getChildWithTypeAndStringAttribute(type,attribute, value, true);
        if (part) return part;
    }
    for (var i in this._projects) {
        if (i == cp) continue; // skip current project, already looked there
        part = this._projects[i].part.getChildWithTypeAndStringAttribute(type,attribute, value, true);
        if (part) return part;
    }
    return null;
}

projectManager.prototype.findPartByAttributeValue= function (attribute, value) {
    var part;
    var cp = this.getCurrentProjectId();
    if (cp >= 0) {
        part = this._projects[cp].part.getChildByAttributeValue(attribute, value, true);
        if (part) return part;
    }
    for (var i in this._projects) {
        if (i == cp) continue; // skip current project, already looked there
        part = this._projects[i].part.getChildByAttributeValue(attribute, value, true);
        if (part) return part;
    }
    return null;
}

projectManager.prototype.getState = function ()
{
    if (this._projects.length == 0) {
        return null; // persist nothing
    }
    // Return a pref to add to the persisted 'workspace'
    var opened_projects = Components.classes['@activestate.com/koOrderedPreference;1'].createInstance();
    opened_projects.id = 'opened_projects';
    var i, project, url;
    for (i = 0; i < this._projects.length; i++) {
        project = this._projects[i];
        url = project.part.url;
        opened_projects.appendStringPref(url);
    }
    return opened_projects;
}


projectManager.prototype.setState = function (pref)
{
    try {
        var i, file_url;
        // Load projects indicated in the pref
        for (i=0; i < pref.length; i++) {
            file_url = pref.getStringPref(i);
            // skip opening of recently opened files -- that's taken care of
            // by the view persistence
            project_openProjectFromURL(file_url, true);
        }
    } catch (e) {
        this.log.exception(e);
    }
}

projectManager.prototype.writeable = function () {
    // The project may not be writeable, but it can
    // be saved as...  so we'll always allow destructive or other
    // 'changing' operations'
    return true;
}
//-------------------------------------------------------------------------
// command implementations
//


function project_openProjectFromURL(url, skipRecentOpenFeature /* false */) {
    var action = null;
    var opened_files = [];
    if (typeof(skipRecentOpenFeature) == 'undefined') {
        skipRecentOpenFeature = false;
    }
    if (!gWorkspaceRestoreInProgress) {
        // another part of the workspace restoration will show the tab if necessary
        uilayout_ensureTabShown('project_tab');
    }

    var viewStateMRU = gPrefSvc.getPrefs("viewStateMRU");
    if (!skipRecentOpenFeature && viewStateMRU.hasPref(url)) {
        var projectViewState = viewStateMRU.getPref(url);
        if (projectViewState.hasPref('opened_files')) {
            opened_files = projectViewState.getPref('opened_files');
            if (opened_files.length > 0) {
                action = dialog_yesNoCancel(
                    "Would you like to open files that you last had open "+
                        "in this project?",
                    "Yes", null, null, // default response, text, title
                    "open_recent_files_on_project_open");
                if (action == "Cancel") {
                    return;
                }
            }
        }
    }
    var project = gProjectManager.loadProject(url)
    if (gCodeIntelActive) {
        // We delay launching the code browser for a little while to provide a
        // smoother project-opening experience. It's not a big deal that the
        // code browser gets populated a tad late.
        window.setTimeout(
            function(codeIntelSvc, msg, url, project) {
                codeIntelSvc.ideEvent(msg, url, project);
            },
            1000, gCodeIntelSvc, "opened_project", url, project
        );
    }
    if (action == "Yes") {
        var v, file_url;
        for (var i=0; i < opened_files.length; i++) {
            file_url = opened_files.getStringPref(i);
            v = gViewMgr.getViewForFile(file_url);
            if (v == null) { // don't re-open existing open filed -- it slows things down.
                open_openURI(file_url);
            }
        }
    }
}

function ProjectSaveAs(project)
{
    var localPath = filepicker_saveFile(
            null, project.url, // default dir and filename
            "Save Project As", // title
            "Komodo Project", // default filter name
            ["Komodo Project", "All"]); // filter names to show
    if (localPath == null) {
        return false;
    }
    var url = uriparse_localPathToURI(localPath);

    if (gProjectManager.getProjectByURL(url) != null) {
        dialog_alert("Sorry, but project " + url + " is already loaded.");
        return false;
    }

    project.url = project.part.url = url;
    project.name = project.part.name = uriparse_baseName(url);
    try {
        project.part.save();
    } catch(ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        dialog_alert('There was an error saving project "' +
              project.name + '": ' +
              lastErrorSvc.getLastErrorMessage());
        return false;
    }

    // Update the MRU projects list.
    MRU_addURL("mruProjectList", url);
    try {
        gObserverSvc.notifyObservers(this,'file_changed', project.url);
    } catch(e) { /* exception if no listeners */ }
    window.updateCommands('project_dirty');
    return true;
}

var gFileStatusSvc = null;
function fileStatusServiceOnLoad()
{
    // start the file status service
    gFileStatusSvc = Components.classes["@activestate.com/koFileStatusService;1"].getService(Components.interfaces.koIFileStatusService);
    gFileStatusSvc.init();
}

function projectManagerOnLoad()
{
    gTimeline.enter('fileStatusServiceOnLoad');
    gTimeline.startTimer('fileStatusServiceOnLoad');
    fileStatusServiceOnLoad();
    gTimeline.stopTimer('fileStatusServiceOnLoad');
    gTimeline.markTimer('fileStatusServiceOnLoad');
    gTimeline.leave('fileStatusServiceOnLoad');
    gTimeline.enter('projectExtensionManager');
    gTimeline.startTimer('extensionManager');
    extensionManager = new projectExtensionManager();
    extensionManager.init();
    gTimeline.stopTimer('extensionManager');
    gTimeline.markTimer('extensionManager');
    gTimeline.leave('projectExtensionManager');
    gTimeline.startTimer('projectManager');
    gProjectManager = new projectManager();
    gTimeline.stopTimer('projectManager');
    gTimeline.markTimer('projectManager');
}

function projectManagerCanUnload()
{
    return true;
}

function projectManagerOnUnload()
{
    projectManagerCanUnload();
    gFileStatusSvc.unload();
}


























































































/*
 * provide support functions for accessing prefs from projects or views
 *
 * the functions should implement the following selection rules:
 *
 * file is in project(s)
 *    prefs are ALWAYS retrieved from project part of the url is in
 *    a project.  First the current project is looked at, then all
 *    projects are looked at, the first found used.
 * file is in toolbox
 *    if a url is NOT in project(s), then the toolbox part prefs are used
 * file is open, but not in projects or toolbox
 *    prefs are stored into global prefs.  The pref functions
 *    in this file should not be used to store 'view' prefs which
 *    include such items as tab width, font, etc.
 *
 *
 * To be loaded in the same .xul file as projectviewer.js
 */

const EFFECTIVE_PREFS_VIEW = 0x01; // use viewprefs if available
const EFFECTIVE_PREFS_TEMP = 0x02; // use temporary pref rather than global
const EFFECTIVE_PREFS_TOOLBOX = 0x04; // use toolbox pref if available
const EFFECTIVE_PREFS_PROJECT = 0x08; // use project pref if available
const EFFECTIVE_PREFS_ALL = 0xFF; // use any prefs, but not global prefs

var gUrlPrefCache = {};

function getEffectivePrefsForFile(url, flags)
{
    if (typeof(flags) == 'undefined' || !flags)
        flags = EFFECTIVE_PREFS_ALL;
    //dump('find prefset for '+url+'\n');
    // see if part is in the project manager
    // the project manager looks in the current project first
    if (url) {
        var part;
        if (flags & EFFECTIVE_PREFS_PROJECT) {
            part = gProjectManager.findPartByTypeAttributeValue('file','url',url,true);
            if (part) {
                return part.prefset;
            }
        }
        if (flags & EFFECTIVE_PREFS_TOOLBOX) {
            // see if the part is in the toolbox
            part = toolboxMgr.findPartByTypeAttributeValue('file','url',url,true);
            if (part) {
                return part.prefset;
            }
        }
        if (flags & EFFECTIVE_PREFS_VIEW) {
            var v = gViewMgr.getViewForFile(url);
            if (v) {
                return v.prefs;
            }
        }
        if (flags & EFFECTIVE_PREFS_TEMP) {
            if (typeof(gUrlPrefCache[url]) == 'undefined') {
                gUrlPrefCache[url] = Components.classes["@activestate.com/koPreferenceSet;1"]
                    .createInstance(Components.interfaces.koIPreferenceSet);
                gUrlPrefCache[url].parent = gPrefs;
            }
            return gUrlPrefCache[url];
        }
    }
    if (flags & EFFECTIVE_PREFS_TEMP) {
        // build a throw-away prefset
        //dump('returning temporary prefset\n');
        var prefset = Components.classes["@activestate.com/koPreferenceSet;1"]
            .createInstance(Components.interfaces.koIPreferenceSet);
        prefset.parent = gPrefs;
        return prefset;
    }
    // prefs are global
    return gPrefs;
}
