/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */


var wizardManager = null;
var wd_info = null;
var _gOs = Components.classes["@activestate.com/koOs;1"].createInstance();
var _gPhpInfo = Components.classes["@activestate.com/koPHPInfoInstance;1"].createInstance();
var _gOsPath = Components.classes["@activestate.com/koOsPath;1"].getService();
var _gInfoSvc = Components.classes["@activestate.com/koInfoService;1"].getService();

var _gFindingInterps = false;
var existingIni=null;
var currentDir=null;

//---- interface routines for wsSetupWizard.xul

function onWizardLoad()
{
    // set wizard data for the panels
    wd_info = window.arguments[0];
}

function onWizardCancel()
{
    // First, ensure that the user wants to cancel.
    var wantToCancel = dialog_yesNo("Are you sure you want to cancel "
                                    + "installing PHP Debugger support?",
                                    "No");    // default answer

    // Second, go to the "Warning" panel.
    if (wantToCancel == "Yes") {
        wd_info.ok = false;
        return true;
    }
    return false;
}


function _chooseExecutable_InterpPathToInstallPath(interpreterPath)
{
    var installationPath = null;
    if (interpreterPath) {
        if (_gInfoSvc.platform.substring(0,3) == "win") {
            installationPath = _gOsPath.dirname(interpreterPath);
        } else {
            installationPath = _gOsPath.dirname(_gOsPath.dirname(
                interpreterPath));
        }
    }
    return installationPath;
}


function _chooseExecutable_UpdateInstallation(installation)
{
    // Persist.
    wd_info.interpreter = installation;

    // Display the installation selection.
    var installWidget = document.getElementById("installation");
    if (installation == null) {
        installWidget.value = "";
    } else {
        installWidget.value = installation;
    }
}



//---- Wizard Infrastructure interface routines

function chooseExecutable_validate()
{
    if (wd_info.interpreter &&
        _gOsPath.exists(wd_info.interpreter)) {
        _gPhpInfo.executablePath = wd_info.interpreter;
        if (_gPhpInfo.valid_version) return true;
        dialog_alert("The version of the selected PHP executable "+
              "("+_gPhpInfo.version+") is not supported by Komodo\n");
    }
    return false;
}


//---- interface for wsSetup_ChooseInstallation.xul

function chooseExecutable_onPageShow()
{
    chooseExecutable_UpdateInterpreterList()
    _chooseExecutable_UpdateInstallation(wd_info.interpreter);
}

function chooseExecutable_UpdateInterpreterList()
{
    // setup "avail-install-list"
    var listWidget = document.getElementById("avail-install-list");
    // - remove any existing items and add a "finding..." one
    _gFindingInterps = true;
    while (listWidget.hasChildNodes()) {
        listWidget.removeChild(listWidget.childNodes[0]);
    }
    listWidget.appendItem("Finding available PHP installation...");
    // - get a list of installations
    var sysUtils = Components.classes['@activestate.com/koSysUtils;1'].
                   getService(Components.interfaces.koISysUtils);

    // get a list of installed PHP interpreters
    var numFound = new Object();
    var availInterps = _gPhpInfo.FindInstallationExecutables(numFound);

    // - populate the tree listing them
    listWidget.removeChild(listWidget.childNodes[0]); // "Finding..."
    if (availInterps.length != 0) {
        for (var i = 0; i < availInterps.length; i++) {
            listWidget.appendItem(availInterps[i]);
        }
    }
    _gFindingInterps = false;

    // If there is no installation setting or a new language was
    // selected then pick a reasonable default.
    if (!wd_info.interpreter) {
        wd_info.interpreter = _gPhpInfo.executablePath;
    }
    _chooseExecutable_UpdateInstallation(wd_info.interpreter);
}


function chooseExecutable_UpdateInstallation_OnSelection()
{
    // User selected one of the "found installations".
    if (! _gFindingInterps) {
        // Load the default installation textfield with the selection
        var listbox = document.getElementById("avail-install-list");
        _chooseExecutable_UpdateInstallation(listbox.selectedItem.getAttribute('label'));
    }
}


function chooseExecutable_UpdateInstallation_OnKey()
{
    // User typed into the installation field.
    // - Unselect any tree item.
    var listbox = document.getElementById("avail-install-list");
    listbox.selectedIndex = -1;

    // - Persist the value.
    var installWidget = document.getElementById("installation");
    _chooseExecutable_UpdateInstallation(installWidget.value);
}


function chooseExecutable_UpdateInstallation_OnBrowse()
{
    try {
        var current = document.getElementById('installation').value;
        var php = filepicker_openExeFile(null, current);
        if (php != null) {
            _chooseExecutable_UpdateInstallation(php);
        }
    } catch(e) {
        dump(e);
    }
}

function ini_validate()
{
    wd_info.existingIni = document.getElementById("existingini").value;
    wd_info.ini = document.getElementById("debuginifile").value;
    try {
        if (!wd_info.existingIni || !wd_info.ini) {
            dialog_alert("You need to select an ini file and a destination for the debug version of the ini file.");
            return false;
        }
        if (!_gOsPath.isfile(wd_info.existingIni)) {
            dialog_alert("The ini file you selected to copy is not a valid file.");
            return false;
        }
        if (_gOsPath.isfile(wd_info.ini)) {
            dialog_alert("The destination must be a directory.");
            return false;
        }
        if (!_gOsPath.isdir(wd_info.ini)) {
            dialog_alert("You have choosen a destination for the debug ini file that does not exist.");
            return false;
        }
        if (!_gOs.saneAccessParent(wd_info.ini, Components.interfaces.koIOs.W_OK)) {
            dialog_alert("You do not have access to write to the ini directory you have choosen.");
            return false;
        }
        return true;
    } catch(e) {
        dump(e);
    }
    return false;
}

function _ini_UpdateExistingIni(ini)
{
    // Persist.
    existingIni = wd_info.existingIni = ini;

    // Display the installation selection.
    document.getElementById("existingini").value = ini;
}

function _ini_UpdateDebuggerIni(ini)
{
    // Persist.
    wd_info.ini = _gOsPath.realpath(ini);

    // Display the installation selection.
    document.getElementById("debuginifile").value = ini;
}

function ini_onPageShow()
{
    _gPhpInfo.executablePath = wd_info.interpreter
    _ini_UpdateExistingIni(_gPhpInfo.cfg_file_path);

    var ini = _gOsPath.join(_gOsPath.dirname(wd_info.interpreter),'debug');
    if (wd_info.ini) ini = wd_info.ini;
    _ini_UpdateDebuggerIni(ini)
}

function ini_UpdateExistingIni_OnBrowse()
{
    var defaultDir=null, defaultFilename=null;
    if (existingIni) {
        defaultDir = null;
        defaultFilename = existingIni;
    } else {
        defaultDir = null; //XXX Could do better to suggest dir here.
        defaultFilename = "php.ini";
    }
    var ini = filepicker_openFile(
                defaultDir,
                defaultFilename,// default filename
                "Choose an INI file to copy from", //title
                "INI", // default filter
                ["INI", "All"]); // filters
    if (ini != null) {
        _ini_UpdateExistingIni(ini);
    }
}

function ini_UpdateDebuggerIni_OnBrowse()
{
    var dir = document.getElementById("debuginifile").value;
    var inipath = filepicker_getFolder(dir);
    if (inipath != null) {
        _ini_UpdateDebuggerIni(inipath);
    }
}

function extdir_validate()
{
    wd_info.extensionDir = document.getElementById("extensiondir").value;
    if (wd_info.extensionDir && !_gOs.saneAccessParent(wd_info.extensionDir, Components.interfaces.koIOs.W_OK)) {
        dialog_alert("You do not have access to write to the extension directory you have choosen.");
        return false;
    }
    return wd_info.extensionDir &&
        wd_info.extensionDir == _gOsPath.realpath(wd_info.extensionDir)
}

function _extdir_UpdateExistingDir(dir)
{
    // Persist.
    currentDir = dir;
    if (!wd_info.extensionDir)
        wd_info.extensionDir = dir;

    // Display the installation selection.
    document.getElementById("existingextdir").value = dir;
}

function _extdir_UpdateExtensionDir(dir)
{
    // Persist.
    wd_info.extensionDir = _gOsPath.realpath(dir);

    // Display the installation selection.
    document.getElementById("extensiondir").value = dir;
}

function extdir_onPageShow()
{
    _gPhpInfo.installationPath = wd_info.interpreter;
    // use the ini we're going to copy
    _gPhpInfo.cfg_file_path = _gOsPath.dirname(wd_info.ini);
    _extdir_UpdateExistingDir(_gPhpInfo.extension_dir);

    var dir = wd_info.extensionDir;
    // an unconfigured ini file with have ./
    if (!dir || dir == './') {
        dir = _gOsPath.join(_gOsPath.dirname(wd_info.interpreter),'extensions');
    }
    _extdir_UpdateExtensionDir(dir)
}

function extdir_UpdateDir_OnBrowse()
{
    var dir = document.getElementById("extensiondir").value;
    if (!dir) {
    }
    var extensionDir = filepicker_getFolder(dir)
    if (extensionDir != null) {
        _extdir_UpdateExtensionDir(extensionDir);
    }
}


function install_onPageShow()
{
    // Describe the installation procedure.
    var actionWidget = document.getElementById("php-executable");
    var description = 'PHP Executable: '+parent.wd_info.interpreter;
    var textNode = document.createTextNode(description);
    while (actionWidget.firstChild) {
        actionWidget.removeChild(actionWidget.firstChild)
    }
    actionWidget.appendChild(textNode);

    actionWidget = document.getElementById("php-inipath");
    while (actionWidget.firstChild) {
        actionWidget.removeChild(actionWidget.firstChild)
    }
    description = 'PHP INI Path: '+parent.wd_info.ini;
    textNode = document.createTextNode(description);
    actionWidget.appendChild(textNode);

    actionWidget = document.getElementById("php-extensions");
    while (actionWidget.firstChild) {
        actionWidget.removeChild(actionWidget.firstChild)
    }
    description = 'PHP Extension Directory: '+parent.wd_info.extensionDir;
    textNode = document.createTextNode(description);
    actionWidget.appendChild(textNode);
}


function _installing_Install()
{
    var msg=null;
    // XXX If this is a long process then should really add a progress
    //     meter, or perhaps stream in the PyPPM output as it comes.
    SetBusyCursor(window, true);
    // do it now
    try {
        var configure = Components.classes["@activestate.com/koPHPConfigurator;1"].
                createInstance(Components.interfaces.koIPHPConfigurator);

        msg = configure.configureDebugger(
                                wd_info.interpreter,
                                wd_info.existingIni,
                                wd_info.ini,
                                wd_info.extensionDir
                                )
    } catch(e) {
        dump(e);
        msg = "An Internal error has occurred!";
    }
    SetBusyCursor(window, false);

    // Determine if successful.
    wd_info.msg = msg;
    if (!msg) {
        wd_info.ok = true;
    } else {
        wd_info.ok = false;
    }
}



//---- interface for wsSetup_Installing.xul

function installing_onPageShow()
{
    // <description id="please-wait-msg"/>
    var pleaseWaitMsg = document.getElementById("please-wait-msg");
    var text = "Please wait while Komodo sets up your "
               + "PHP installation for Debugging. "
    while (pleaseWaitMsg.firstChild) {
        pleaseWaitMsg.removeChild(pleaseWaitMsg.firstChild)
    }
    pleaseWaitMsg.appendChild(document.createTextNode(text));

    // Setup for WS.
    var wizard = document.getElementById("php-install-wizard-window");
    wizard.canAdvance = false;
    wizard.canRewind = false;
    try {
        _installing_Install();
    } catch(e) {
        dump(e);
        wd_info.msg = "An Internal error has occurred!";
        wd_info.ok = false;
    }
    wizard.canAdvance = true;
    wizard.advance(null);
}


function finished_onPageShow()
{
    var wizard = document.getElementById("php-install-wizard-window");
    wizard.getButton('cancel').setAttribute('collapsed','true');
    wizard.getButton('back').setAttribute('collapsed','true');
    wizard.getButton('next').setAttribute('collapsed','true');
    var donemsg = document.getElementById("finish-msg");
    var text;
    if (parent.wd_info.ok) {
        text = "PHP Debugging has been successfully configured."
    } else {
        text = "Sorry, PHP Debugging configuration failed. "+parent.wd_info.msg+
            "  Please consult the help file for more information on configuring "+
            "PHP.  If you still have problems, contact technical support at "+
            "support@ActiveState.com.";
    }
    while (donemsg.firstChild) {
        donemsg.removeChild(donemsg.firstChild)
    }

    donemsg.appendChild(document.createTextNode(text));
}
