/* Copyright (c) 2000-2001 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */
var gPDKInfo = null;
var gPDKGuiInfo = null;

function PDKInfoInit() {
    if (!gPDKInfo) {
        gPDKInfo = new PDKInfo('pdkLocation','pdkGuiLocation','perlapp','perlfb');
        gPDKGuiInfo = new PDKInfo('pdkGuiLocation','pdkLocation','perlfb','perlapp');
        if (!gPDKInfo.isWindows) {
            // remove apps not on unix
            var el = document.getElementById('popup_pdk');
            // popup_pdk will only exist in the top level window.  When the
            // prefs window uses pdkinfo, this will fail.
            if (!el) return;
            el.removeChild(document.getElementById('menu_pdkPerlSvc'));
            el.removeChild(document.getElementById('menu_pdkPerlCtrl'));
            el.removeChild(document.getElementById('menu_pdkPerlTray'));
            el.removeChild(document.getElementById('menu_pdkPerlVBConverter'));
            el = document.getElementById('pdkToolbar');
            el.removeChild(document.getElementById('tb_pdkPerlSvc'));
            el.removeChild(document.getElementById('tb_pdkPerlCtrl'));
            el.removeChild(document.getElementById('tb_pdkPerlTray'));
            el.removeChild(document.getElementById('tb_pdkPerlVBConverter'));
        }
    }
}

function PDKInfo(primaryPrefName, fallbackPrefName, primaryExeName, fallbackExeName) {
    this.log = getLoggingMgr().getLogger("PDK");
    //this.log.setLevel(LOG_DEBUG);
    this.log.debug("PDKItemsController init");
    this.primaryPrefName = primaryPrefName;
    this.primaryExeName = primaryExeName;
    this.fallbackPrefName = fallbackPrefName;
    this.fallbackExeName = fallbackExeName;
    this._cache = {};
    try {
        window.controllers.appendController(this);
        this.infoService = Components.classes['@activestate.com/koInfoService;1'].
                getService(Components.interfaces.koIInfoService);
        this.isWindows = this.infoService.platform.substring(0,3) == "win";

        this.osPathSvc = Components.classes["@activestate.com/koOsPath;1"].
                getService(Components.interfaces.koIOsPath);
        this.koDirSvc = Components.classes["@activestate.com/koDirs;1"].
                getService(Components.interfaces.koIDirs);
        this.perlInfo = Components.classes["@activestate.com/koPerlInfoEx;1"].
                getService(Components.interfaces.koIPerlInfoEx);
        this.sysUtils = Components.classes["@activestate.com/koSysUtils;1"].
            getService(Components.interfaces.koISysUtils);
        this.prefs = Components.classes["@activestate.com/koPrefService;1"].
                    getService(Components.interfaces.koIPrefService).prefs;
        this.prefs.prefObserverService.addObserver(this, primaryPrefName, false);
    } catch(e) {
        this.log.exception(e);
    }
}

PDKInfo.prototype = {
    getLocation: function() {
        var location = null;
        if (typeof(this._cache['location']) != 'undefined') {
            return this._cache['location'];
        }
        try {
            if (this.prefs.hasStringPref(this.primaryPrefName)) {
                location = this.prefs.getStringPref(this.primaryPrefName);
            }
            else if (this.prefs.hasStringPref(this.fallbackPrefName)) {
                location = this.prefs.getStringPref(this.fallbackPrefName);
            }
            if (!location) {
                var availInstalls = this.installationPaths();
                if (availInstalls.length > 0)
                    location = availInstalls[0];
            }
            if (!location) return null;
        } catch(e) {
            this.log.exception(e);
        }
        this._cache['location'] = location;
        return location;
    },
    
    getAppPath: function(app) {
        if (typeof(this._cache[app]) != 'undefined' &&
            typeof(this._cache[app]['path']) != 'undefined') {
            return this._cache[app]['path'];
        }
        try {
            this._cache[app] = {};
            this._cache[app]['path'] = this.appPath(this.getLocation(), app);
            return this._cache[app]['path'];
        } catch(e) {
            this.log.exception(e);
        }
        return null;
    },
    
    appPath: function(location, app) {
        if (!location) return null;
        try {
            // given an app name (eg. perlapp), build the path
            if (this.isWindows) {
                app += ".exe";
            }
            return this.osPathSvc.joinlist(3, [location, "bin", app]);
        } catch(e) {
            this.log.exception(e);
        }
        return null;
    },
    
    getPAIPath: function()
    {
        try {
            return this.PAIPath(this.getLocation());
        } catch(e) {
            this.log.exception(e);
        }
        return null;
    },

    PAIPath: function(location)
    {
        if (!location) return null;
        try {
            if (this.isWindows) {
                return this.osPathSvc.joinlist(4, [location, "bin", "lib", "pai.exe"]);
            }
            return this.osPathSvc.joinlist(3, [location, "lib", "pai"]);
        } catch(e) {
            this.log.exception(e);
        }
        return null;
    },

    hasApp: function (app) {
        try {
            if (typeof(this._cache[app]) != 'undefined' &&
                typeof(this._cache[app]['exists']) != 'undefined' &&
                this._cache[app]['exists']) {
                return true;
            }
            var app_path = this.getAppPath(app);
            if (!app_path) return false;
            this._cache[app]['exists'] =  this.osPathSvc.exists(app_path);
            return this._cache[app]['exists'];
        } catch(e) {
            this.log.exception(e);
        }
        return false;
    },
    
    exists: function (location) {
        try {
            return this.osPathSvc.exists(this.PAIPath(location));
        } catch(e) {
            this.log.exception(e);
        }
        return false;
    },
    
    _BaseDir: function(location)
    {
        // pdk puts executables in pdkdir/bin on all platforms
        // pdkDir/bin/someapp.exe
        try {
            return this.osPathSvc.dirname(this.osPathSvc.dirname(location));
        } catch(e) {
            this.log.exception(e);
        }
        return null;
    },
    
    installationPaths: function()
    {
        var availInterps = new Array();
        try {
            var found = this.sysUtils.WhichAll(this.primaryExeName, new Object());
            var found2 = this.sysUtils.WhichAll(this.fallbackExeName, new Object());
            found = found.concat(found2);

            var dir = null;
            for (var i=0; i < found.length; i++) {
                dir = this._BaseDir(found[i]);
                availInterps[availInterps.length] = dir;
                
            }
        } catch(ex) {
            log.exception(ex);
        }
        return availInterps;
    },

    hasLicense: function() {
        try {
            if (typeof(this._cache['license']) != 'undefined' &&
                this._cache['license']) return true;
            
            var sysUtilsSvc = Components.classes["@activestate.com/koSysUtils;1"].
                              getService(Components.interfaces.koISysUtils);
            var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
                          getService(Components.interfaces.koIInfoService);
            var interp = gPrefs.getStringPref("perlDefaultInterpreter");
            if (interp == '') {
                interp = 'perl';
                interp = sysUtilsSvc.Which(interp);
                if (!interp) {
                    dialog_alert("This feature requires that you have " +
                          "Perl on your PATH or that the Perl default " +
                          "interpreter preference be correctly " +
                          "set, and that a licensed copy of Perl " +
                          "Dev Kit 6.0 or later be installed in " +
                          "the same directory.");
                }
            }
            var perlInfoEx = Components.classes["@activestate.com/koPerlInfoEx;1"].createInstance(Components.interfaces.koIPerlInfoEx);
            perlInfoEx.installationPath = perlInfoEx.getInstallationPathFromBinary(interp);
            var sysDataSvc = Components.classes["@activestate.com/koSystemDataService;1"]
                             .getService(Components.interfaces.koISystemDataService);
    
            var perlBinDir = null;
            if (gPrefs.hasStringPref(this.primaryPrefName)) {
                // hack in the pdkLocation for the old pdk dialog, this enables
                // support of different installations in the old dialog
                perlBinDir = gPrefs.getStringPref(this.primaryPrefName);
                if (perlBinDir) {
                    perlBinDir = os.path.join(perlBinDir,'bin');
                }
            }
            if (!perlBinDir) {
                // is pdk 6 on the path?
                var paths = this.installationPaths();
                if (paths.length) {
                    perlBinDir = os.path.join(paths[0],'bin');
                } else {
                    perlBinDir = os.path.dirname(interp);
                }
            }
            sysDataSvc.setString("pdk.perlbinpath", perlBinDir);
    
            if (! sysDataSvc.getString('pdk.perlapp')) {
                dialog_alert("Komodo could not find an installation of Perl Dev Kit in: '" + perlBinDir + "'.");
                return false;
            }
            if (! sysDataSvc.getBoolean("pdk.haveLicense")) {
                dialog_alert("This feature requires that you have " +
                      "a license for the Perl Dev Kit installed.");
                return false;
            }
            var version = parseFloat(sysDataSvc.getString("pdk.version"));
            if (!version) {
                dialog_alert("Komodo could not determine the version number for Perl Dev Kit at ["+perlBinDir+"].");
                return false;
            }
            if (version < 6) {
                dialog_alert("Komodo requires Perl Dev Kit version 6 or later.");
                return false;
            }
            this._cache['license'] = true;
            return true;
        } catch (e) {
            log.error(e);
        }
        return false;
    },

    observe: function(subject, topic, data)
    {
        this.log.debug('observe '+topic);
        try {
            switch (topic) {
            case this.primaryPrefName:
                this._cache = {};
                window.updateCommands("pdk_prefchange");
            }
        } catch(e) {
            this.log.exception(e);
        }
    }

}
