# Copyright (c) 2000-2001 ActiveState Corporation.
# See the file LICENSE.txt for licensing information.

# these classes implement support for the dbgp uri protocol via
# the standard mozilla method of adding new network uri protocols

import urllib
import urlparse
import re
import xpcom
from xpcom import components, ServerException, nsError

from StringIO import StringIO
import logging
import koUnicodeEncoding, types

_scheme = "dbgp"
log = logging.getLogger("dbgp.ProtocolHandler")


class koSimpleStringInputStream:
    _com_interfaces_ = [components.interfaces.nsIInputStream,
                        components.interfaces.koIInputStreamHack]

    nonBlocking=0

    def __init__(self, data):
        # find out what the proper encoding is for this buffer
        encodingServices = components.classes['@activestate.com/koEncodingServices;1'].\
                 getService(components.interfaces.koIEncodingServices)
        prefs = components.classes["@activestate.com/koPrefService;1"].\
                            getService(components.interfaces.koIPrefService).prefs
        if type(data)==types.UnicodeType:
            data = data.encode('UTF-8')
        encoding_name = prefs.getStringPref('encodingDefault')
        encoding = encodingServices.get_encoding_info(encoding_name).python_encoding_name
        unicodebuffer, encoding, bom =\
            koUnicodeEncoding.autoDetectEncoding(data, 1, 1, 1, defaultEncoding=encoding)
        # we encode to the encoding now, koDocument will handle things
        # from there
        self.data=StringIO(unicodebuffer.encode(encoding))

    def close( self ):
        pass
    def available( self ):
        return self.data.len-self.data.pos

    def readStr( self, amount):
        return self.data.read(amount)

    read=readStr


class koSimpleStringChannel:
    """Note that this class only implements what we need in the short-term.
    Weird stuff like asynch can wait. """
    _com_interfaces_ = [components.interfaces.nsIChannel]

    def __init__(self, url, data):
        self.uri=url
        self.name=url
        self.originalURI=url
        self.data=data

    def open(self):
        return koSimpleStringInputStream(self.data)

    def asyncOpen(self):
        return None

class koDBGPURI:
    _com_interfaces_ = [components.interfaces.koIDBGPURI]
    _reg_clsid_ = "{4A207FA1-A178-48B2-B10D-795C0D863BC3}"
    _reg_contractid_ = "@activestate.com/koDBGPURI;1"

    scheme=_scheme
    
    def init(self, uri):
        if uri.startswith('dbgp:'):
            self.spec= uri
            self.path = uri.split(':',1)[1]
        else:
            self.path = uri
            self.spec= "%s:%s" % (_scheme, uri)
        return 

    def newURI(self, uri):
        "Create a new one based on this one."
        #Unobfuscation: generates dbgp:uri
        newobj = self.__class__()
        if uri.startswith('dbgp:'):
            newobj.init(uri)
        else:
            newobj.init("%s:%s" % (_scheme, uri))
        return newobj

    def getBaseURI(self, uri):
        return uri

class koDBGPProtocolHandler:
    _com_interfaces_ = [components.interfaces.koIDBGPProtocolHandler]
    _reg_clsid_ = "{43441B03-528A-44B8-9D30-957E8BAAA65F}"
    _reg_contractid_ = "@mozilla.org/network/protocol;1?name=%s"%_scheme

    scheme=_scheme

    defaultPort=9000
    instantiated=0
    counter=0
    protocolFlags = 0
    
    def __init__(self):
        self._manager = None
        assert koDBGPProtocolHandler.instantiated==0, "Should be singleton"
        koDBGPProtocolHandler.instantiated=1

    def setUIManager(self, manager):
        self._manager = manager

    def newURI(self, newuri, aOriginCharset, base):
        if base is None:
            base = koDBGPURI()
            base.init(newuri)
            return base
        else:
            return base.newURI(newuri)

    def newChannel(self, url):
        # get the current debugger session
        # XXX this may be flaky getting the right uri to the right
        # session
        app = self._manager.getCurrentDebugApplication()
        # if the spec contains a file spec, then we want to return that to
        # the debug engine instead.  We may want to eventualy expand this to
        # other specs as well, but for now file is enough for remote debugging
        if url.spec[8:].startswith('file:/'):
            # remove the dbgp spec identifier and send the original file spec
            return koSimpleStringChannel(url, app.currentSession.getSourceCode(url.spec[8:], 0, 0))
        else:
            return koSimpleStringChannel(url, app.currentSession.getSourceCode(url.spec, 0, 0))

