from xpcom import components, ServerException

from koLanguageServiceBase import *

def registerLanguage(registery):
    registery.registerLanguage(koJavaScriptLanguage())

class koJavaScriptLanguage(KoLanguageBase):
    name = "JavaScript"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{0DD83914-E2F6-478a-A77E-7307833A7601}"

    accessKey = 'j'
    primary = 1
    defaultExtension = ".js"
    commentDelimiterInfo = {
        "line": [ "//" ],
        "block": [ ("/*", "*/") ],
        "markup": "*",
    }
    _dedenting_statements = [u'throw', u'return', u'break', u'continue']
    _indenting_statements = [u'case']

    _block_comment_styles = [sci_constants.SCE_C_COMMENT,
                            sci_constants.SCE_C_COMMENTDOC,
                            sci_constants.SCE_C_COMMENTDOCKEYWORD,
                            sci_constants.SCE_C_COMMENTDOCKEYWORDERROR]
    # matches:
    # function name
    # name: function
    # name = function
    # class.prototype.name = function
    namedBlockDescription = 'JavaScript functions and classes'
    namedBlockRE = r'^[ |\t]*?(?:([\w|\.|_]*?)\s*=\s*function|function\s*([\w|\_]*?)|([\w|\_]*?)\s*:\s*function).*?$'
    supportsSmartIndent = "brace"
    sample = """function chkrange(elem,minval,maxval) {
    // Comment
    if (elem.value < minval - 1 ||
        elem.value > maxval + 1) {
          alert("Value of " + elem.name + " is out of range!");
    }
}
"""
    
    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_CPP)
            self._lexer.setKeywords(0, self._keywords)
            self._lexer.supportsFolding = 1
        return self._lexer

    def get_linter(self):
        if self._linter is None:
            self._linter = KoLinterLanguageService("@activestate.com/koXPCShellLinter;1")
        return self._linter

    _keywords = ["abstract", "boolean", "break", "byte", "case", "catch",
                 "char", "class", "const", "continue", "debugger", "default",
                 "delete", "do", "double", "else", "enum", "export",
                 "extends", "false", "final", "finally", "float", "for", "function",
                 "goto", "if", "implements", "import", "in", "instanceof",
                 "int", "interface", "long", "native", "new", "null", "package",
                 "private", "protected", "public", "return", "short",
                 "static", "super", "switch", "synchronized", "this", "throw",
                 "throws", "transient", "true", "try", "typeof", "var", "void",
                 "while", "with"]
